'From Squeak 2.4 of April 21, 1999 on 23 April 1999 at 11:30:52 am'!
"Change Set:		privateGraphics-sw
Date:			23 April 1999
Author:			Scott Wallace

Provides mechanisms for stripping private graphics from the StandardScriptingSystem's FormDictionary and storing them to a file, and for restoring such graphics from just such a file.

Two general methods for deleting and restoring graphics are provided, and two custom methods specific to needs of the moment are also present.

The latter two deal with the simplified case of storing/restoring the graphics to/from a file in the default directory, and they use a hard-coded list of names of pictures to be deleted, and a hard-coded file name.  These are called as follows:
	ScriptingSystem deletePrivateGraphics.
	ScriptingSystem restorePrivateGraphics.

The deletePrivateGraphics method will obliterate the private disney graphics from the image, and in the process will create a file named 'disGraphics' in the default directory on disc, which will hold all the private graphics.

The restorePrivateGraphics method will merge all the graphics found in a file named 'disGraphics' (in the default directory) back into the system's FormDictionary.  (That file needs to be in the appropriate format, of course.)"!


!StandardScriptingSystem methodsFor: 'form dictionary' stamp: 'sw 4/23/1999 11:18'!
deletePrivateGraphics
	"ScriptingSystem deletePrivateGraphics"
	self deletePrivateGraphics:
			#('BadgeMiniPic' 'BadgePic' 'Broom' 'CedarPic' 'CollagePic' 'CoverMain' 'CoverSpiral' 'CoverTexture' 'Fred' 'ImagiPic' 'KayaPic' 'StudioPic')
		afterStoringToFileNamed: 'disGraphics'! !

!StandardScriptingSystem methodsFor: 'form dictionary' stamp: 'sw 4/23/1999 11:15'!
deletePrivateGraphics: nameList afterStoringToFileNamed: aFileName
	"This method is used to strip private graphics from the FormDictionary and store them on a file of the given name"

	|  replacement toRemove aReferenceStream |
	toRemove _ Dictionary new.
	replacement _ FormDictionary at: 'Gets'.

	nameList do:
		[:aKey |
			(toRemove at: aKey put: (FormDictionary at: aKey)).
			FormDictionary at: aKey put: replacement].

	aReferenceStream _ ReferenceStream fileNamed: aFileName.
	aReferenceStream nextPut: toRemove.
	aReferenceStream close! !

!StandardScriptingSystem methodsFor: 'form dictionary' stamp: 'sw 4/23/1999 11:11'!
mergeGraphicsFrom: aDictionary
	"aDictionary is assumed to hold associations of the form <formName> -> <form>.   Merge the graphics held by that dictionary into the internal FormDictionary, overlaying any existing entries with the ones found in aDictionary"

	aDictionary associationsDo:
		[:assoc | FormDictionary at: assoc key put: assoc value]! !

!StandardScriptingSystem methodsFor: 'form dictionary' stamp: 'sw 4/23/1999 11:24'!
restorePrivateGraphics
	"ScriptingSystem restorePrivateGraphics"
	| aReferenceStream |
	aReferenceStream _ ReferenceStream fileNamed: 'disGraphics'.
	self mergeGraphicsFrom: aReferenceStream next.
	aReferenceStream close.
! !


