'From Squeak 2.4b of April 23, 1999 on 29 June 1999 at 2:49:34 am'!
"Change Set:		WnldUpdates
Date:			29 June 1999
Author:			Jeff Pierce

Changes to Wonderland classes to improve the ActorViewer."!

StringHolder subclass: #WonderlandActorBrowser
	instanceVariableNames: 'myWonderland myListMorph actorListIndex selectedActor actorViewer '
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants '
	category: 'Wonderland-Morphs'!
Morph subclass: #WonderlandActorViewer
	instanceVariableNames: 'thumbnailMorph thumbnailCamera selectedActor '
	classVariableNames: ''
	poolDictionaries: 'WonderlandConstants '
	category: 'Wonderland-Morphs'!

!Hierarchical methodsFor: 'private' stamp: 'jsp 6/29/1999 02:36'!
appearsInChildLists
	"Returns true if this object should appear in the actor browser"

	^ true.! !

!Hierarchical methodsFor: 'private' stamp: 'jsp 6/29/1999 02:37'!
appendChildrenNamesTo: prefix
	"Return the object's children's names, each appended to the prefix."

	| nameList |

	nameList _ OrderedCollection new.
	myChildren do: [:child |
						(child appearsInChildLists) ifTrue: [
							nameList addLast: (prefix , (child getName)).
							nameList _ nameList , (child appendChildrenNamesTo: (prefix , '    '))].
					].
	^ nameList.
! !

!Hierarchical methodsFor: 'parent-child' stamp: 'jsp 6/29/1999 02:43'!
getAllChildren
	"Return all of this instance's children"

	| children |

	children _ OrderedCollection new.

	myChildren do: [:child | 
		(child appearsInChildLists)
			ifTrue: [
						children addLast: child.
						children _ children , (child getAllChildren).
					].
	].

	^ children.! !


!WonderlandActorBrowser methodsFor: 'initialize-release' stamp: 'jsp 6/29/1999 01:50'!
initializeWith: aWonderland
	"Initialize this instance."

	myWonderland _ aWonderland.
	actorListIndex _ 0.
	selectedActor _ nil.
	actorViewer _ nil.

	myListMorph _ (PluggableListMorph on: self list: #actorList
						selected: #actorListIndex changeSelected: #actorListIndex:
						menu: #actorMenu: keystroke: #systemCatListKey:from:).
	myListMorph name: 'Actors'.
	myListMorph scrollBarOnLeft: false.
	myListMorph extent: 140@320.
	myListMorph color: (Color r: 0.627 g: 0.909 b: 0.972).
	myListMorph retractable: false.
! !

!WonderlandActorBrowser methodsFor: 'accessing' stamp: 'jsp 6/29/1999 01:44'!
getSelectedActor
	"Returns the currently selected actor in the actor browser."

	^ selectedActor.
! !

!WonderlandActorBrowser methodsFor: 'accessing' stamp: 'jsp 6/29/1999 01:52'!
setActorViewer: anActorViewer
	"Sets the WonderlandActorViewer to update when the user changes the selected object in the browser"

	actorViewer _ anActorViewer.
! !

!WonderlandActorBrowser methodsFor: 'actor list management' stamp: 'jsp 6/29/1999 01:55'!
actorListIndex: anInteger
	"Set the index of the currently selected actor."

	| scene childList |

	actorListIndex _ anInteger.
	myListMorph selectionIndex: anInteger.

	(anInteger = 0)
		ifTrue: [
					selectedActor _ nil.
				]
		ifFalse: [
					scene _ myWonderland getScene.
					childList _ scene getAllChildren.
					childList addFirst: scene.
					selectedActor _ childList at: anInteger.
				].

	actorViewer ifNotNil: [
		actorViewer setSelectedActor: selectedActor.
	].

	self contentsChanged.
! !


!WonderlandActorViewer reorganize!
('initialization' initialize initializeWith:)
('updating' setSelectedActor:)
!


!WonderlandActorViewer methodsFor: 'initialization' stamp: 'jsp 6/29/1999 01:39'!
initialize
	"Initialize this morph"

	super initialize.

	self name: 'Actor Info'.
	self extent: 500@350.
	self color: (Color r: 0.815 g: 0.972 b: 0.878).
! !

!WonderlandActorViewer methodsFor: 'initialization' stamp: 'jsp 6/29/1999 01:40'!
initializeWith: aWonderland
	"Do the Wonderland specific setup for this viewer."

	thumbnailCamera _ WonderlandStillCamera createFor: aWonderland.
	thumbnailMorph _ thumbnailCamera getMorph.
	self addMorph: thumbnailMorph.
! !

!WonderlandActorViewer methodsFor: 'updating' stamp: 'jsp 6/29/1999 02:13'!
setSelectedActor: anActor
	"Update the actor viewer to reflect the information for this actor"

	selectedActor _ anActor.

	(selectedActor isNil)
		ifTrue: []
		ifFalse: [
			thumbnailCamera moveTo: { -1.5. 0.5. 2.6 } duration: rightNow asSeenBy: anActor.
			thumbnailCamera pointAt: anActor duration: rightNow.
			thumbnailMorph changed.
		].

! !


!WonderlandEditor methodsFor: 'initialization' stamp: 'jsp 6/29/1999 01:54'!
initializeWith: aWonderland
	"Initialize the editor with the Wonderland."

	| hAlign vAlign |
	myWonderland _ aWonderland.

	self extent: 650@375.

	hAlign _ AlignmentMorph new.
	hAlign color: (Color gray).
	hAlign extent: 650@375.
	hAlign hResizing: #rigid.
	self addMorph: hAlign.

	myTabs _ (WonderlandEditorTabs newFor: aWonderland).
	hAlign addMorph: myTabs.

	vAlign _ AlignmentMorph new.
	vAlign color: (Color r: 0.784 g: 0.784 b: 0.784).
	vAlign orientation: #vertical.
	hAlign addMorph: vAlign.

	myActorBrowser _ (WonderlandActorBrowser newFor: myWonderland).
	myActorBrowser setActorViewer: (myTabs getActorViewer).
	vAlign addMorph: (myActorBrowser getMorph).

	myControls _ WonderlandControls newFor: aWonderland.
	vAlign addMorph: myControls.

	self position: 200@20.
	self color: (Color r: 0.784 g: 0.784 b: 0.784).
	self openInWorld.
! !


!WonderlandEditorTabs methodsFor: 'initialization' stamp: 'jsp 6/29/1999 01:41'!
initializeWith: aWonderland
	"Initialize the editor with the Wonderland."

	myWonderland _ aWonderland.

	myScriptEditor _ (WonderlandScriptEditor new).
	self addPanel: (myScriptEditor getMorph).
	(self tabsMorph submorphs at: 1) tabSelected.
	myScriptEditor setBindings: (myWonderland getNamespace).

	myActorViewer _ WonderlandActorViewer new.
	myActorViewer initializeWith: aWonderland.
	self addPanel: myActorViewer.

	myQuickReference _ (WonderlandQuickReference new).
	self addPanel: (myQuickReference getMorph).
! !

!WonderlandEditorTabs methodsFor: 'accessing' stamp: 'jsp 6/29/1999 01:53'!
getActorViewer

	^ myActorViewer.
! !


!WonderlandStillCamera methodsFor: 'initialize-release' stamp: 'jsp 6/29/1999 02:41'!
initializeFor: aWonderland
	"Initializes the camera."

	super initializeFor: aWonderland.

	"Set the camera's mesh and geometry"
	self setMesh: nil.
	self setTexturePointer: nil.

	"Delete the WonderlandCameraMorph our superclass created"
	myMorph delete.	

	"Create a WonderlandStillCameraMorph for the camera to render into"
	myMorph _ WonderlandStillCameraMorph new.
	myMorph initializeWithCamera: self.
	myMorph openInWorld.

	"Initially draw the scene background"
	drawSceneBackground _ true.
! !

!WonderlandStillCamera methodsFor: 'private' stamp: 'jsp 6/29/1999 02:40'!
appearsInChildLists
	"Return false; we want to hide the still camera from the user"

	^ false.
! !


