'From Squeak 2.1 of June 30, 1998 on 20 August 1998 at 9:45:08 am'!
"Change Set:		devEnvMisc-sw
Date:			20 August 1998
Author:			Scott Wallace

Miscellaneous development-environment enhancements and tweaks, including:

* Shows the method author/time-stamp as title to message-list menus, in browsers, debuggers, and change sorters.  Use of this feature can be suppressed by hard-coding the Preference #showTimeStampsInMenuTitles to false.
* Adds an optional debug halo-handle to the halo, which will show or not show depending on the setting of the hard-coded Preference #showDebugHaloHandle.
* Adds a protocol for a PluggableListView and a ScrollPane to obtain a menu-title from its model.
* Adds 'title' inst var to CustomMenu, so that it can be set up before the menu is popped up -- this allows menu titles to be used with the pluggable interfaces.
* Makes Behavior.changeRecordsAt: robust when called for non-existent method. 
* Adds 'revert and forget' and 'remove from current change set' to the shifted message-list menus of debuggers (had formerly been present there, but somehow got lost during the changeover to Pluggables.)
* Fixes a bug in PluggableTest, and enhances it so that it illustrates the menu-title feature.
* Parameterizes the lists of selectors and globals to test for in 'check for slips'.
* Adds a hard-coded Preference governing whether the check-for-slips test should be done.
* Fixes the 'subtract-other-side' feature of change sorter (had been very damagingly broken.)"!

SelectionMenu subclass: #CustomMenu
	instanceVariableNames: 'labels dividers lastDivider title '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Menus'!
ListView subclass: #PluggableListView
	instanceVariableNames: 'getListSelector getSelectionSelector setSelectionSelector getMenuSelector getMenuTitleSelector keystrokeActionSelector autoDeselect '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Pluggable'!
Object subclass: #PluggableTest
	instanceVariableNames: 'musicTypeList musicTypeIndex artistList artistIndex '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Pluggable'!
ComponentLikeModel subclass: #ScrollPane
	instanceVariableNames: 'scrollBar scroller retractableScrollBar scrollBarOnLeft getMenuSelector getMenuTitleSelector scrollBarHidden '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Windows'!

!Behavior methodsFor: 'accessing method dictionary' stamp: 'sw 8/18/1998 16:38'!
changeRecordsAt: selector
	"Return a list of ChangeRecords for all versions of the method at selector. Source code can be retrieved by sending string to any one.  Return nil if the method is absent."

	"(Pen changeRecordsAt: #go:) collect: [:cRec | cRec string]"
	^ (ChangeList new
			scanVersionsOf: (self compiledMethodAt: selector ifAbsent: [^ nil])
			class: self meta: self isMeta
			category: (self whichCategoryIncludesSelector: selector)
			selector: selector)
		changeList! !


!ChangeSet methodsFor: 'change management' stamp: 'sw 8/17/1998 16:05'!
expungeEmptyClassChangeEntries
	| toRemove |
	toRemove _ OrderedCollection new.
	classChanges associationsDo:
		[:assoc | assoc value isEmpty ifTrue:
			[toRemove add: assoc key]].
 	toRemove do:
		[:aKey | classChanges removeKey: aKey ifAbsent: []]! !

!ChangeSet methodsFor: 'change management' stamp: 'sw 8/17/1998 14:36'!
forgetAllChangesFoundIn: aChangeSet
	"Remove from the receiver all method changes found in aChangeSet. The intention is facilitate the process of factoring a large set of changes into disjoint change sets.  To use:  in a change sorter, copy over all the changes you want into some new change set, then use the subtract-other-side feature to subtract those changes from the larger change set, and continue in this manner.
	sw 8/14/1998 17:30:  This method somehow became very badly, and damagingly, broken since I originally wrote it.  Below is an attempt to bring it back to health, or at least to remove the most damaging of its bugs."

	| cls itsMethodChanges myClassChange |
	aChangeSet == self ifTrue: [^ self].

	aChangeSet changedClassNames do: 
		[:className |
			(cls _ Smalltalk classNamed: className) ~~ nil ifTrue:
				[itsMethodChanges _ aChangeSet methodChanges at: className 
						ifAbsent: [Dictionary new].
				itsMethodChanges associationsDo:
					[:assoc | self forgetChange: assoc value forSelector: assoc key class: cls].
				myClassChange _ self classChangeAt: className.
				myClassChange size > 0 ifTrue:
					[(aChangeSet classChangeAt: className) do:
						[:aChange | myClassChange remove: aChange ifAbsent: []]]]].

	classRemoves removeAllFoundIn: aChangeSet classRemoves.	"names of them"
	self expungeEmptyClassChangeEntries
! !

!ChangeSet methodsFor: 'change management' stamp: 'sw 8/14/1998 17:22'!
removeAllMemoryOfChangesTo: class
	"Remove all memory of changes associated with this class"

	classChanges removeKey: class name ifAbsent: [].
	methodChanges removeKey: class name ifAbsent: [].
	classRemoves remove: class name ifAbsent: [].! !

!ChangeSet methodsFor: 'method changes' stamp: 'sw 8/14/1998 17:16'!
forgetChange: actionToSubtract forSelector: selector class: class 
	"If the receiver records a change that can be subsumed by the change given by the parameters, which characterize a change in another change set which is being 'subtracted' from the receiver, remove it from the receiver."

	| myChangeDictionaryForClass methodCurrentlyExists |

	myChangeDictionaryForClass _ methodChanges at: class name ifAbsent: [^ self].
	methodCurrentlyExists _ class includesSelector: selector.

	((#(add change) includes: actionToSubtract) and: [methodCurrentlyExists])
		ifTrue:
			[myChangeDictionaryForClass removeKey: selector ifAbsent: []].

	((#(remove addedThenRemoved) includes: actionToSubtract) and: [methodCurrentlyExists not])
			ifTrue:
				[myChangeDictionaryForClass removeKey: selector ifAbsent: []].

	myChangeDictionaryForClass isEmpty ifTrue: [methodChanges removeKey: class name]! !

!ChangeSet methodsFor: 'fileIn/Out' stamp: 'sw 8/10/1998 07:13'!
checkForSlips
	"Return a collection of method refs with possible debugging code in them."
	| slips changes method |
	slips _ OrderedCollection new.
	self changedClasses do:
		[:aClass |
		changes _ methodChanges at: aClass name ifAbsent: [nil].
		changes ifNotNil:
			[changes associationsDo: 
				[:mAssoc | (#(remove addedThenRemoved) includes: mAssoc value) ifFalse:
					[method _ aClass compiledMethodAt: mAssoc key ifAbsent: [nil].
					method ifNotNil:
						[method hasReportableSlip
							ifTrue: [slips add: aClass name , ' ' , mAssoc key]]]]]].
	^ slips! !

!ChangeSet methodsFor: 'fileIn/Out' stamp: 'sw 8/19/1998 12:18'!
fileOut
	"File out the receiver, to a file whose name is a function of the change-set name and of the date and the time."
	"wod 5/12/1998: use #asFileName rather than #truncateTo:"
	| file slips |
	Cursor write showWhile: [
		file _ FileStream newFileNamed:
			(self name, FileDirectory dot, Utilities dateTimeSuffix, FileDirectory dot, 'cs') asFileName.
		file header; timeStamp.
		self fileOutPreambleOn: file.
		self fileOutOn: file.
		self fileOutPostscriptOn: file.
		file trailer; close].

	Preferences suppressCheckForSlips ifTrue: [^ self].  "Can hard-code that pref if desired"

	slips _ self checkForSlips.
	(slips size > 0 and: [self confirm: 'Methods in this fileOut have halts
or references to the Transcript
or other ''slips'' in them.
Would you like to browse them?'])
		ifTrue: [Smalltalk browseMessageList: slips
							name: 'References to #halt or Transcript']! !


!CompiledMethod methodsFor: 'testing' stamp: 'sw 8/20/1998 09:31'!
hasReportableSlip
	"Answer whether the receiver contains anything that might be brought to the attention of the author when filing out.   Customize the lists to suit your preferences.  If slips do not get reported in spite of your best efforts here, make certain that the Preference 'suppressCheckForSlips' has not been hard-coded to true."

	| assoc | 
	#(halt halt: urgent hottest) do:
		[:aLit | (self hasLiteral: aLit) ifTrue: [^ true]].

	#(Transcript AA BB CC DD EE) do:
		[:aSymbol | (assoc _ (Smalltalk associationAt: aSymbol ifAbsent: [nil])) ifNotNil:
			[(self hasLiteral: assoc) ifTrue: [^ true]]].

	^ false! !


!CustomMenu methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:01'!
title: aTitle
	title _ aTitle! !

!CustomMenu methodsFor: 'invocation' stamp: 'sw 8/18/1998 12:01'!
startUp: initialSelection
	"Build and invoke this menu with the given initial selection. Answer the selection associated with the menu item chosen by the user or nil if none is chosen."

	^ self startUp: initialSelection withCaption: title! !


!MessageSet class methodsFor: 'instance creation' stamp: 'sw 8/18/1998 12:23'!
open: aMessageSet name: aString 
	"Create a standard system view for the messageSet, aMessageSet, whose label is aString."
	| topView aListView aBrowserCodeView |

	World ifNotNil: [^ self openAsMorph: aMessageSet name: aString].

	topView _ (StandardSystemView new) model: aMessageSet.
	topView label: aString.
	topView minimumSize: 180 @ 120.
	topView borderWidth: 1.

	aListView _ PluggableListView on: aMessageSet
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	aListView  menuTitleSelector: #messageListSelectorTitle.
	aListView window: (0 @ 0 extent: 180 @ 100).
	topView addSubView: aListView.

	aBrowserCodeView _ PluggableTextView on: aMessageSet 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	aBrowserCodeView window: (0 @ 0 extent: 180 @ 300).
	topView addSubView: aBrowserCodeView below: aListView.
	topView controller open! !

!MessageSet class methodsFor: 'instance creation' stamp: 'sw 8/19/1998 13:51'!
openAsMorph: aMessageSet name: labelString 
	"Create a SystemWindow aMessageSet, with the label labelString, in a Morphic project"
	^ self openAsMorph: aMessageSet name: labelString inWorld: World! !

!MessageSet class methodsFor: 'instance creation' stamp: 'sw 8/18/1998 17:05'!
openAsMorph: aMessageSet name: labelString inWorld: aWorld
	"Create a SystemWindow aMessageSet, with the label labelString."
	| window aListMorph |
	self flag: #newBrowsers.
	window _ (SystemWindow labelled: labelString) model: aMessageSet.
	aListMorph _ PluggableListMorph on: aMessageSet list: #messageList
			selected: #messageListIndex changeSelected: #messageListIndex:
			menu: #messageListMenu:shifted:
			keystroke: #messageListKey:from:.
	aListMorph menuTitleSelector: #messageListSelectorTitle.
	window addMorph: aListMorph
		frame: (0@0 extent: 1@0.2).

	window addMorph: (PluggableTextMorph on: aMessageSet 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:)
		frame: (0@0.2 corner: 1@1).

	window openInWorld: aWorld! !


!Morph methodsFor: 'accessing' stamp: 'sw 8/19/1998 11:55'!
balloonHelpTextForHandle: aHandle
	|  itsSelector |
	itsSelector _ aHandle eventHandler firstMouseSelector.
	#(	(startRot:with:				'Rotate')
		(startGrow:with:				'Change size') 
		(doDup:with:				'Duplicate')
		(maybeDoDup:with:			'Duplicate')
		(doGrab:with:				'Pick up')
		(startDrag:with:				'Move')
		(doMenu:with:				'Menu')
		(doDebug:with:				'Debug')
		(mouseDownOnHelpHandle:	'Help')
		(tearOffTile					'Make a Tile')
		(paintBackground			'Paint background')
		(openViewerForArgument	'View me')
		(dismiss						'Remove')
		(editDrawing				'Repaint')
		(startScale:with:				'Change scale') 
		(chooseFont					'Change font')
		(chooseStyle					'Change style')
		(chooseEmphasis				'Change emphasis')
		(makeNewDrawingWithin	'Paint new object'))  "This last one not currently in use"


	do:
		[:pair | itsSelector == pair first ifTrue: [^ pair last]].
	^ 'unknown halo handle'! !

!Morph methodsFor: 'apr98 additions' stamp: 'sw 8/19/1998 11:52'!
addDebuggingItemsTo: aMenu hand: aHandMorph
	aMenu add: 'debug...' subMenu:  (self debuggingMenuFor: aHandMorph)! !

!Morph methodsFor: 'apr98 additions' stamp: 'sw 8/19/1998 11:51'!
debuggingMenuFor: aHandMorph
	| aMenu |
	aMenu _ MenuMorph new defaultTarget: aHandMorph.

	aMenu add: 'control-menu...' target: aHandMorph selector: #invokeMetaMenuFor: argument: aHandMorph argument.
	aMenu add: 'inspect morph' action: #inspectArgument.
	World ifNil:
		[aMenu add: 'inspect morph (morphic)' action: #inspectArgumentInMorphic].
	aMenu add: 'browse morph class' action: #browseMorphClass.

	self player ifNotNil:
		[aMenu add: 'inspect player' action: #inspectPlayer.
		World ifNil: [aMenu add: 'inspect player (morphic)' action: #inspectArgumentsPlayerInMorphic].
		aMenu add: 'browse player class' action: #browsePlayerClass].

	aMenu add: 'make own subclass' action: #subclassMorph.
	aMenu add: 'internal name' action: #nameMorph.
	aMenu add: 'save morph in file' action: #saveOnFile.

	aMenu defaultTarget: self.
	aMenu add: 'edit balloon help' action: #editBalloonHelpText.
	aMenu add: 'temp command' action: #tempCommand.
	^ aMenu! !


!HaloMorph methodsFor: 'private' stamp: 'sw 8/19/1998 11:58'!
addHandles
	| box s |
	target isWorldMorph ifTrue: [^ self addHandlesForWorldHalos].
	self removeAllMorphs.  "remove old handles, if any"
	s _ self handleSize.
	self bounds: target fullBoundsInWorld.  "update my size"
	box _ (self fullBounds expandBy: s+1)
			intersect: (self world bounds insetBy: 5@5).

	self addHandleAt: box topLeft color: Color red
		on: #mouseDown send: #doMenu:with: to: self.

	self addHandleAt: (box leftCenter + (0 @ (s+2)) min: box leftCenter + box bottomLeft // 2)
		color: Color lightBrown
		on: #mouseDown send: #tearOffTile to: innerTarget.

	self addHandleAt: (box topLeft + ((s+2)@0) min: box topLeft + box topCenter // 2)
		color: Color transparent
		on: #mouseDown send: #dismiss to: self.
	self addHandleAt: (box leftCenter) color: Color cyan
		on: #mouseDown send: #openViewerForArgument to: innerTarget.

	self addHandleAt: box topCenter color: Color black
		on: #mouseDown send: #doGrab:with: to: self.

	(self addHandleAt: (box topCenter + ((s+2)@0) min: box topCenter + box topRight // 2)
		color: Color brown
		on: #mouseDown send: #startDrag:with: to: self)
		on: #mouseStillDown send: #doDrag:with: to: self.

	self addHandleAt: box topRight color: Color green
		on: #mouseDown send: #doDup:with: to: self.

	Preferences showDebugHaloHandle ifTrue:
		[self addHandleAt: (box rightCenter + (0 @ ((s+2) negated)) min: box rightCenter + box bottomRight // 2)
			color: Color blue veryMuchLighter
			on: #mouseDown send: #doDebug:with: to: self].

	target balloonText ifNotNil:
		[(self addHandleAt: box bottomCenter color: Color lightBlue
			on: #mouseDown send: #mouseDownOnHelpHandle: to: innerTarget)
			on: #mouseUp send: #deleteBalloon to: innerTarget].

	(self addHandleAt: box bottomLeft color: Color blue
		on: #mouseDown send: #startRot:with: to: self)
		on: #mouseStillDown send: #doRot:with: to: self.

	target isFlexMorph
		ifTrue: [(self addHandleAt: box bottomRight color: Color lightOrange
				on: #mouseDown send: #startScale:with: to: self)
				on: #mouseStillDown send: #doScale:with: to: self]
		ifFalse: [(self addHandleAt: box bottomRight color: Color yellow
				on: #mouseDown send: #startGrow:with: to: self)
				on: #mouseStillDown send: #doGrow:with: to: self].

	innerTarget addOptionalHandlesTo: self box: box.
	self addNameBeneath: box string: target externalName.
	growingOrRotating _ false.
	self layoutChanged.
	self changed.
! !

!HaloMorph methodsFor: 'private' stamp: 'sw 8/19/1998 11:46'!
doDebug: evt with: menuHandle
	"Ask hand to invoke the a debugging menu for my inner target."

	| menu |
	self removeAllHandlesBut: nil.  "remove all handles"
	self world doOneCycle.
	menu _ evt hand buildDebugHandleMenuFor: innerTarget.
	menu addTitle: innerTarget externalName.
	evt hand invokeMenu: menu event: evt.
! !


!HandMorph methodsFor: 'meta menu' stamp: 'sw 8/19/1998 11:54'!
buildDebugHandleMenuFor: argMorph
	"Build the menu for the given morph's halo's debug handle."
	argument _ argMorph.
	^ argMorph debuggingMenuFor: self! !


!PluggableListView methodsFor: 'initialization' stamp: 'sw 8/18/1998 12:04'!
menuTitleSelector: getMenuTitleSel
	getMenuTitleSelector _ getMenuTitleSel! !

!PluggableListView methodsFor: 'model access' stamp: 'sw 8/18/1998 12:07'!
getMenu: shiftKeyDown
	"Answer the menu for this text view, supplying an empty menu to be filled in. If the menu selector takes an extra argument, pass in the current state of the shift key."
	| menu aMenu |
	getMenuSelector == nil ifTrue: [^ nil].
	menu _ CustomMenu new.
	getMenuSelector numArgs = 1
		ifTrue:
			[aMenu _ model perform: getMenuSelector with: menu.
			getMenuTitleSelector ifNotNil: [aMenu title: (model perform: getMenuTitleSelector)].
			^ aMenu].
	getMenuSelector numArgs = 2
		ifTrue: [aMenu _ model perform: getMenuSelector with: menu with: shiftKeyDown.
				getMenuTitleSelector ifNotNil: [aMenu title: (model perform: getMenuTitleSelector)].
				^ aMenu].
	^ self error: 'The getMenuSelector must be a 1- or 2-keyword symbol'! !


!PluggableTest methodsFor: 'music type pane' stamp: 'sw 8/18/1998 12:31'!
musicTypeListTitle
	^ 'Choose a command'! !

!PluggableTest methodsFor: 'menu commands' stamp: 'sw 8/18/1998 12:29'!
perform: sel orSendTo: otherObject
	(self respondsTo: sel) ifTrue: [self perform: sel] ifFalse: [otherObject perform: sel]! !


!PluggableTest class methodsFor: 'example' stamp: 'sw 8/18/1998 12:25'!
open
	"PluggableTest open"

	| model listView1 topView listView2 |
	model _ self new initialize.
	listView1 _
		PluggableListView on: model
			list: #musicTypeList
			selected: #musicType
			changeSelected: #musicType:
			menu: #musicTypeMenu:
			keystroke: #musicTypeKeystroke:.
	listView1  menuTitleSelector: #musicTypeListTitle.
	listView2 _
		PluggableListView on: model
			list: #artistList
			selected: #artist
			changeSelected: #artist:
			menu: nil
			keystroke: #artistKeystroke:.

	topView _ StandardSystemView new
		label: 'Pluggable Test';
		minimumSize: 300@200;
		borderWidth: 1;
		addSubView: listView1;
		addSubView: listView2 toRightOf: listView1.
	topView borderWidth: 1.
	topView controller open.! !


!Preferences class methodsFor: 'hard-coded preferences' stamp: 'sw 8/19/1998 13:22'!
showDebugHaloHandle
	"Manually set this to false if you do not want the debug handle to show"
	^ true! !

!Preferences class methodsFor: 'hard-coded preferences' stamp: 'sw 8/19/1998 12:16'!
showTimeStampsInMenuTitles
	"Set this manually to false if you don't want time stamps to show in menu titles of message-list panes"
	^ true! !

!Preferences class methodsFor: 'hard-coded preferences' stamp: 'sw 8/20/1998 09:22'!
suppressCheckForSlips
	"Manually set this to true if you don't want every file-out-changes to check for slips"
	^ false! !


!ScrollPane methodsFor: 'menu' stamp: 'sw 8/18/1998 12:40'!
getMenu: shiftKeyState
	"Answer the menu for this text view, supplying an empty menu to be filled in. If the menu selector takes an extra argument, pass in the current state of the shift key."
	| menu aMenu |
	getMenuSelector == nil ifTrue: [^ nil].
	menu _ MenuMorph new defaultTarget: model.
	getMenuSelector numArgs = 1 ifTrue:
		[aMenu _ model perform: getMenuSelector with: menu.
		getMenuTitleSelector ifNotNil: [aMenu addTitle: (model perform: getMenuTitleSelector)].
		^ aMenu].
	getMenuSelector numArgs = 2 ifTrue:
		[aMenu _ model perform: getMenuSelector with: menu with: shiftKeyState.
		getMenuTitleSelector ifNotNil: [aMenu addTitle: (model perform: getMenuTitleSelector)].
		^ aMenu].
	^ self error: 'The getMenuSelector must be a 1- or 2-keyword symbol'! !

!ScrollPane methodsFor: 'menu' stamp: 'sw 8/18/1998 12:38'!
menuTitleSelector: aSelector
	getMenuTitleSelector _ aSelector! !


!StringHolder methodsFor: 'message list menu' stamp: 'sw 8/14/1998 06:51'!
copyName
	"Copy the current selector to the clipboard"
	| selector |
	(selector _ self selectedMessageName) ifNotNil:
		[ParagraphEditor new clipboardTextPut: selector asString asText]! !

!StringHolder methodsFor: 'message list menu' stamp: 'sw 8/19/1998 12:15'!
messageListSelectorTitle
	| class selector aString aStamp |
	Preferences showTimeStampsInMenuTitles
		ifFalse:
			[^ nil].
	(selector _ self selectedMessageName) ifNotNil:
		[class _ self selectedClassOrMetaClass.
		aString _ class name, '.', selector.
		^ (aStamp _ self timeStamp) ifNotNil:
				[aString, String cr, aStamp]
			ifNil:
				[aString]].
	^ nil! !

!StringHolder methodsFor: 'message list menu' stamp: 'sw 8/18/1998 12:16'!
timeStamp
	| class selector recordList |
	(selector _ self selectedMessageName) ifNotNil:
		[class _ self selectedClassOrMetaClass.
		(recordList _ class changeRecordsAt: selector) size > 0
			ifTrue:
				[^ recordList first stamp]].
	^ nil! !


!Browser methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:50'!
openAsMorphEditing: editString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	| window codePane aListMorph |
	window _ (SystemWindow labelled: 'later') model: self.

	window addMorph: (PluggableListMorph on: self list: #systemCategoryList
			selected: #systemCategoryListIndex changeSelected: #systemCategoryListIndex:
			menu: #systemCategoryMenu:)
		frame: (0@0 extent: 0.25@0.4).
	window addMorph: (PluggableListMorph on: self list: #classList
			selected: #classListIndex changeSelected: #classListIndex:
			menu: #classListMenu:)
		frame: (0.25@0 extent: 0.25@0.3).
	window addMorph: self buildMorphicSwitches
		frame: (0.25@0.3 extent: 0.25@0.1).
	window addMorph: (PluggableListMorph on: self list: #messageCategoryList
			selected: #messageCategoryListIndex changeSelected: #messageCategoryListIndex:
			menu: #messageCategoryMenu:)
		frame: (0.5@0 extent: 0.25@0.4).
	aListMorph _ PluggableListMorph on: self list: #messageList
			selected: #messageListIndex changeSelected: #messageListIndex:
			menu: #messageListMenu:shifted:.
	aListMorph menuTitleSelector: #messageListSelectorTitle.
	window addMorph: aListMorph
		frame: (0.75@0 extent: 0.25@0.4).

	codePane _ PluggableTextMorph on: self text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	editString ifNotNil: [codePane editString: editString.
					codePane hasUnacceptedEdits: true].
	window addMorph: codePane
		frame: (0@0.4 corner: 1@1).

	^ window! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:08'!
openEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	| systemCategoryListView classListView 
	messageCategoryListView messageListView browserCodeView topView switchView |

	World ifNotNil: [^ self openAsMorphEditing: aString].
	Sensor leftShiftDown ifTrue: [^ self openAsMorphEditing: aString "testing"].

	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	systemCategoryListView _ PluggableListView on: self
		list: #systemCategoryList
		selected: #systemCategoryListIndex
		changeSelected: #systemCategoryListIndex:
		menu: #systemCategoryMenu:.
	systemCategoryListView window: (0 @ 0 extent: 50 @ 70).
	topView addSubView: systemCategoryListView.

	classListView _ PluggableListView on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: #classListMenu:.
	classListView window: (0 @ 0 extent: 50 @ 62).
	topView addSubView: classListView toRightOf: systemCategoryListView.

	switchView _ self buildInstanceClassSwitchView.
	switchView borderWidth: 1.
	topView addSubView: switchView below: classListView.

	messageCategoryListView _ PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 50 @ 70).
	topView addSubView: messageCategoryListView toRightOf: classListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView window: (0 @ 0 extent: 50 @ 70).
	messageListView menuTitleSelector: #messageListSelectorTitle.
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@110).
	topView addSubView: browserCodeView below: systemCategoryListView.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	^ topView
! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:17'!
openMessageCatEditString: aString
	"Create a pluggable version of the views for a Browser that just shows one message category."
	| messageCategoryListView messageListView browserCodeView topView |

	World ifNotNil: [^ self openAsMorphMsgCatEditing: aString].

	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	messageCategoryListView _ PluggableListView on: self
		list: #messageCatListSingleton
		selected: #indexIsOne 
		changeSelected: #indexIsOne:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 200 @ 12).
	topView addSubView: messageCategoryListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView menuTitleSelector: #messageListSelectorTitle.
	messageListView window: (0 @ 0 extent: 200 @ 70).
	topView addSubView: messageListView below: messageCategoryListView.

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@(200-12-70)).
	topView addSubView: browserCodeView below: messageListView.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	^ topView
! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:18'!
openOnClassWithEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers."
	| classListView messageCategoryListView messageListView browserCodeView topView switchView |

	World ifNotNil: [^ self openAsMorphClassEditing: aString].

	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	classListView _ PluggableListView on: self
		list: #classListSingleton
		selected: #indexIsOne 
		changeSelected: #indexIsOne:
		menu: #classListMenu:.
	classListView window: (0 @ 0 extent: 100 @ 12).
	topView addSubView: classListView.

	messageCategoryListView _ PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 100 @ 70).
	topView addSubView: messageCategoryListView below: classListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView menuTitleSelector: #messageListSelectorTitle.
	messageListView window: (0 @ 0 extent: 100 @ 70).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	switchView _ self buildInstanceClassSwitchView.
	switchView borderWidth: 1.
	switchView 
		window: switchView window 
		viewport: (classListView viewport topRight 
					corner: messageListView viewport topRight).
	topView addSubView: switchView toRightOf: classListView.

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@(200-12-70)).
	topView addSubView: browserCodeView below: messageCategoryListView.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	^ topView
! !

!Browser methodsFor: 'initialize-release' stamp: 'sw 8/18/1998 12:18'!
openSystemCatEditString: aString
	"Create a pluggable version of all the views for a Browser, including views and controllers.  The top list view is of the currently selected system class category--a single item list."
	| systemCategoryListView classListView messageCategoryListView messageListView browserCodeView topView switchView |

	World ifNotNil: [^ self openAsMorphSysCatEditing: aString].

	topView _ (StandardSystemView new) model: self.
	topView borderWidth: 1.
		"label and minSize taken care of by caller"

	systemCategoryListView _ PluggableListView on: self
		list: #systemCategorySingleton
		selected: #indexIsOne 
		changeSelected: #indexIsOne:
		menu: #systemCategoryMenu:.
	systemCategoryListView window: (0 @ 0 extent: 200 @ 12).
	topView addSubView: systemCategoryListView.

	classListView _ PluggableListView on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: #classListMenu:.
	classListView window: (0 @ 0 extent: 67 @ 62).
	topView addSubView: classListView below: systemCategoryListView.

	messageCategoryListView _ PluggableListView on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:.
	messageCategoryListView window: (0 @ 0 extent: 66 @ 70).
	topView addSubView: messageCategoryListView toRightOf: classListView.

	switchView _ self buildInstanceClassSwitchView.
	switchView 
		window: switchView window 
		viewport: (classListView viewport bottomLeft 
					corner: messageCategoryListView viewport bottomLeft).
	switchView borderWidth: 1.
	topView addSubView: switchView below: classListView.

	messageListView _ PluggableListView on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:.
	messageListView menuTitleSelector: #messageListSelectorTitle.
	messageListView window: (0 @ 0 extent: 67 @ 70).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	browserCodeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	browserCodeView window: (0@0 extent: 200@(110-12)).
	topView addSubView: browserCodeView below: switchView.
	aString ifNotNil: [browserCodeView editString: aString.
			browserCodeView hasUnacceptedEdits: true].
	^ topView
! !


!ChangeSorter methodsFor: 'creation' stamp: 'sw 8/18/1998 12:43'!
oldOpenAsMorphIn: window rect: rect
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 0.5@0."
	| buttonView col aListMorph |
	contents _ ''.
	self addDependent: window.		"so it will get changed: #relabel"
	buttonView _ PluggableButtonMorph
		on: self
		getState: #mainButtonState
		action: #changeSetMenuStart
		label: #mainButtonName
		menu: #changeSetMenu:.
	col _ Color perform: self defaultBackgroundColor.
	buttonView
		label: myChangeSet name; 
		onColor: col offColor: col;
		triggerOnMouseDown: true; borderColor: window color.
	window addMorph: buttonView
		frame: (((0@0 extent: 1.0@0.06) scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableListMorphByItem on: self
			list: #classList
			selected: #currentClassName
			changeSelected: #currentClassName:
			menu: #classMenu:)
		frame: (((0@0.06 extent: 0.5@0.3) scaleBy: rect extent) translateBy: rect origin).

	aListMorph _ PluggableListMorphByItem on: self
			list: #messageList
			selected: #currentSelector
			changeSelected: #currentSelector:
			menu: #messageMenu:shifted:
			keystroke: #messageListKey:from:.
	aListMorph menuTitleSelector: #messageListSelectorTitle.
	window addMorph: aListMorph
		frame: (((0.5@0.06 extent: 0.5@0.3) scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableTextMorph on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:)
		frame: (((0@0.36 corner: 1@1) scaleBy: rect extent) translateBy: rect origin).
! !

!ChangeSorter methodsFor: 'creation' stamp: 'sw 8/18/1998 16:48'!
openAsMorphIn: window rect: rect
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 0.5@0."

	| chgSetList aListMorph |
	contents _ ''.
	self addDependent: window.		"so it will get changed: #relabel"
	window addMorph: (chgSetList _ PluggableListMorphByItem on: self
			list: #changeSetList
			selected: #currentCngSet
			changeSelected: #showChangeSetNamed:
			menu: #changeSetMenu:)
		frame: (((0@0 extent: 0.5@0.25) scaleBy: rect extent) translateBy: rect origin).
	chgSetList autoDeselect: false.

	window addMorph: (PluggableListMorphByItem on: self
			list: #classList
			selected: #currentClassName
			changeSelected: #currentClassName:
			menu: #classMenu:)
		frame: (((0.5@0 extent: 0.5@0.25) scaleBy: rect extent) translateBy: rect origin).

	aListMorph _ PluggableListMorphByItem on: self
			list: #messageList
			selected: #currentSelector
			changeSelected: #currentSelector:
			menu: #messageMenu:shifted:
			keystroke: #messageListKey:from:.
	aListMorph  menuTitleSelector: #messageListSelectorTitle.
	window addMorph: aListMorph
		frame: (((0@0.25 extent: 1@0.25) scaleBy: rect extent) translateBy: rect origin).

	window addMorph: (PluggableTextMorph on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:)
		frame: (((0@0.5 corner: 1@1) scaleBy: rect extent) translateBy: rect origin).
! !

!ChangeSorter methodsFor: 'creation' stamp: 'sw 8/18/1998 12:18'!
openView: topView offsetBy: offset
	"Add a set of change sorter views to the given top view offset by the given amount. To create a single change sorter, call this once with an offset of 0@0. To create a dual change sorter, call it twice with offsets of 0@0 and 360@0."
	| classView messageView codeView cngSetListView |
	contents _ ''.
	self addDependent: topView.		"so it will get changed: #relabel"
	cngSetListView _ PluggableListViewByItem on: self
		list: #changeSetList
		selected: #currentCngSet
		changeSelected: #showChangeSetNamed:
		menu: #changeSetMenu:.
	cngSetListView window: ((0 @ 0 extent: 180 @ 100) translateBy: offset).
	topView addSubView: cngSetListView.

	classView _ PluggableListViewByItem on: self
		list: #classList
		selected: #currentClassName
		changeSelected: #currentClassName:
		menu: #classMenu:.
	classView window: (180 @ 0 extent: 180 @ 100).
	topView addSubView: classView toRightOf: cngSetListView.

	messageView _ PluggableListViewByItem on: self
		list: #messageList
		selected: #currentSelector
		changeSelected: #currentSelector:
		menu: #messageMenu:shifted:
		keystroke: #messageListKey:from:.
	messageView menuTitleSelector: #messageListSelectorTitle.
	messageView window: (0 @ 100 extent: 360 @ 100).
	topView addSubView: messageView below: cngSetListView.

	codeView _ PluggableTextView on: self 
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
	codeView window: (0 @ 0 extent: 360 @ 180).
	topView addSubView: codeView below: messageView.! !

!ChangeSorter methodsFor: 'class list' stamp: 'sw 8/14/1998 17:22'!
forgetClass
	"Remove all mention of this class from the changeSet"

	self okToChange ifFalse: [^ self].
	currentClassName ifNotNil: [
		myChangeSet removeAllMemoryOfChangesTo: self selectedClassOrMetaClass.
		currentClassName _ nil.
		currentSelector _ nil.
		self showChangeSet: myChangeSet].
! !


!Debugger methodsFor: 'initialize' stamp: 'sw 8/18/1998 12:22'!
buildMVCDebuggerViewLabel: aString minSize: aPoint

	| topView stackListView stackCodeView rcvrVarView rcvrValView ctxtVarView ctxtValView |
	self expandStack.
	topView _ StandardSystemView new model: self.
	topView borderWidth: 1.
	stackListView _ PluggableListView on: self
			list: #contextStackList
			selected: #contextStackIndex
			changeSelected: #toggleContextStackIndex:
			menu: #contextStackMenu:shifted:
			keystroke: #contextStackKey:from:.
		stackListView menuTitleSelector: #messageListSelectorTitle.
		stackListView window: (0 @ 0 extent: 150 @ 50).
		topView addSubView: stackListView.
	stackCodeView _ PluggableTextView on: self
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
		stackCodeView window: (0 @ 0 extent: 150 @ 75).
		topView addSubView: stackCodeView below: stackListView.
	rcvrVarView _ PluggableListView on: self receiverInspector
			list: #fieldList
			selected: #selectionIndex
			changeSelected: #toggleIndex:
			menu: #fieldListMenu:
			keystroke: #inspectorKey:from:.
		rcvrVarView window: (0 @ 0 extent: 25 @ 50).
		topView addSubView: rcvrVarView below: stackCodeView.
	rcvrValView _ PluggableTextView on: self receiverInspector
			text: #contents accept: #accept:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
		rcvrValView window: (0 @ 0 extent: 50 @ 50).
		topView addSubView: rcvrValView toRightOf: rcvrVarView.
	ctxtVarView _ PluggableListView on: self contextVariablesInspector
			list: #fieldList
			selected: #selectionIndex
			changeSelected: #toggleIndex:
			menu: #fieldListMenu:
			keystroke: #inspectorKey:from:.
		ctxtVarView window: (0 @ 0 extent: 25 @ 50).
		topView addSubView: ctxtVarView toRightOf: rcvrValView.
	ctxtValView _ PluggableTextView on: self contextVariablesInspector
			text: #contents accept: #accept:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:.
		ctxtValView window: (0 @ 0 extent: 50 @ 50).
		topView addSubView: ctxtValView toRightOf: ctxtVarView.
	topView label: aString.
	topView minimumSize: aPoint.
	^ topView
! !

!Debugger methodsFor: 'initialize' stamp: 'sw 8/18/1998 12:44'!
openFullMorphicLabel: labelString
	| window aListMorph |
	self expandStack.
	window _ (SystemWindow labelled: labelString) model: self.
	aListMorph _ PluggableListMorph on: self list: #contextStackList
			selected: #contextStackIndex changeSelected: #toggleContextStackIndex:
			menu: #contextStackMenu:shifted: keystroke: #contextStackKey:from:.
	aListMorph menuTitleSelector: #messageListSelectorTitle.
	window addMorph: aListMorph
		frame: (0@0 corner: 1@0.3).
	window addMorph: (PluggableTextMorph on: self
			text: #contents accept: #contents:notifying:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:)
		frame: (0@0.3 corner: 1@0.7).
	window addMorph: (PluggableListMorph on: self receiverInspector list: #fieldList
			selected: #selectionIndex changeSelected: #toggleIndex:
			menu: #fieldListMenu: keystroke: #inspectorKey:from:)
		frame: (0@0.7 corner: 0.2@1).
	window addMorph: (PluggableTextMorph on: self receiverInspector
			text: #contents accept: #accept:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:)
		frame: (0.2@0.7 corner: 0.5@1).
	window addMorph: (PluggableListMorph on: self contextVariablesInspector list: #fieldList
			selected: #selectionIndex changeSelected: #toggleIndex:
			menu: #fieldListMenu: keystroke: #inspectorKey:from:)
		frame: (0.5@0.7 corner: 0.7@1).
	window addMorph: (PluggableTextMorph on: self contextVariablesInspector
			text: #contents accept: #accept:
			readSelection: #contentsSelection menu: #codePaneMenu:shifted:)
		frame: (0.7@0.7 corner: 1@1).

	^ window openInWorld
! !

!Debugger methodsFor: 'context stack menu' stamp: 'sw 8/18/1998 16:35'!
contextStackMenu: aMenu shifted: shifted
	^ shifted ifFalse: [aMenu labels: 
'fullStack (f)
restart (r)
proceed (p)
step (t)
send (e)
where (w)
senders of...
implementors of...
method inheritance
versions
inst var refs...
inst var defs...
class var refs...
class variables
class refs
browse full
more...'
	lines: #(6 10 12 15)
	selections: #(fullStack restart proceed step send where
browseSendersOfMessages browseMessages methodHierarchy browseVersions
browseInstVarRefs browseInstVarDefs
browseClassVarRefs browseClassVariables browseClassRefs
browseMethodFull
shiftedYellowButtonActivity)]

	ifTrue: [aMenu labels: 
'browse class hierarchy
browse class
browse method
implementors of sent messages
change sets with this method
inspect instances
inspect subinstances
revert to previous version
remove from current change set
revert and forget
more...' 
	lines: #(5 7 10)
	selections: #(classHierarchy browseClass 
		buildMessageBrowser browseAllMessages findMethodInChangeSets 
		inspectInstances inspectSubInstances
		revertToPreviousVersion 
		removeFromCurrentChanges revertAndForget
		unshiftedYellowButtonActivity)]

! !


ChangeSet removeSelector: #removeClassChanges:!
