'From Squeak2.9alpha of 12 June 2000 [latest update: #3002] on 12 November 2000 at 5:48:50 pm'!
"Change Set:		OwnerTraversalCleanup
Date:			12 November 2000
Author:			Dan Iingalls

Adds a couple of convenience methods for owner traversal to Morph, and rewrites a dozen methods to use them.

The new methods are...
	<Morph>allOwnersSuchThat: conditionBlock do: aBlock
		'Stops when owner == nil'
	<Morph>firstOwnerSuchThat: conditionBlock
		'Returns the morph or nil' 	<Morph>withOwnersSuchThat: conditionBlock do: aBlock
		'Does self as well as all owners' 	<Morph>orOwnerSuchThat: conditionBlock
		Includes self and returns the morph or nil 
"!


!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:21'!
allOwnersSuchThat: conditionBlock do: aBlock
	| morph |
	morph _ owner.
	[morph == nil] whileFalse:
		[(conditionBlock value: morph) ifTrue: [aBlock value: morph].
		morph _ morph owner].
! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:21'!
firstOwnerSuchThat: conditionBlock

	self allOwnersSuchThat: conditionBlock do: [:m | ^ m].
	^ nil
! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:13'!
nearestOwnerThat: conditionBlock
	"Return the first enclosing morph for which aBlock evaluates to true, or nil if none"

	^ self firstOwnerSuchThat: conditionBlock
! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:22'!
orOwnerSuchThat: conditionBlock

	(conditionBlock value: self) ifTrue: [^ self].
	self allOwnersSuchThat: conditionBlock do: [:m | ^ m].
	^ nil

! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:17'!
outermostMorphThat: aBlock
	"Return the outermost containing morph for which aBlock is true, or nil if none"

	| outermost |
	self allOwnersSuchThat: [:m | aBlock value: m] do: [:m | outermost _ m].
	^ outermost! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:18'!
ownerThatIsA: aClass
	"Return the first enclosing morph that is a kind of aClass, or nil if none"

	^ self firstOwnerSuchThat: [:m | m isKindOf: aClass]! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:20'!
ownerThatIsA: firstClass orA: secondClass
	"Return the first enclosing morph that is a kind of one of the two classes given, or nil if none"

	^ self firstOwnerSuchThat: [:m | (m isKindOf: firstClass) or: [m isKindOf: secondClass]]! !

!Morph methodsFor: 'structure' stamp: 'di 11/12/2000 16:23'!
withOwnersSuchThat: conditionBlock do: aBlock

	(conditionBlock value: self) ifTrue: [aBlock value: self].
	self allOwnersSuchThat: conditionBlock do: aBlock
! !

!Morph methodsFor: 'parts bin' stamp: 'di 11/12/2000 16:10'!
inPartsBin

	self isPartsDonor ifTrue: [^ true].
	self allOwnersSuchThat: [:m | m isPartsBin] do: [:m | ^ true].
	^ false
! !


!CompoundTileMorph methodsFor: 'access' stamp: 'di 11/12/2000 15:31'!
enclosingEditor
	"Return the next scriptor outward in the containment hierarchy"
	^ self firstOwnerSuchThat:
		[:m | (m isKindOf: ScriptEditorMorph) or: [m isKindOf: CompoundTileMorph]]! !


!FlashMorph methodsFor: 'accessing' stamp: 'di 11/12/2000 15:53'!
flashPlayer
	^ self firstOwnerSuchThat: [:parent | parent isFlashMorph and: [parent isFlashPlayer]]! !


!FlashButtonMorph methodsFor: 'accessing' stamp: 'di 11/12/2000 15:59'!
ownerSprite
	"Return the sprite owning the receiver.
	The owning sprite is responsible for executing
	the actions associated with the button."
	^ self orOwnerSuchThat: [:sprite | sprite isFlashMorph and: [sprite isFlashSprite]]! !


!FlashSpriteMorph methodsFor: 'private' stamp: 'di 11/12/2000 16:05'!
getWebBrowser
	"Return a web browser if we're running in one"

	self withOwnersSuchThat: [:m | true]
		do: [:morph | morph isWebBrowser ifTrue: [^ morph].
			(morph hasProperty: #webBrowserView) ifTrue: [^ morph model]].
	^ nil! !


!PhraseTileMorph methodsFor: 'mouse' stamp: 'di 11/12/2000 15:31'!
enclosingEditor
	"Return the next scriptor outward in the containment hierarchy"
	^ self firstOwnerSuchThat:
		[:m | (m isKindOf: ScriptEditorMorph) or: [m isKindOf: CompoundTileMorph]]! !


!ScriptEditorMorph methodsFor: 'private' stamp: 'di 11/12/2000 15:32'!
enclosingEditor
	"Return the next scriptor outward in the containment hierarchy"
	^ self firstOwnerSuchThat:
		[:m | (m isKindOf: ScriptEditorMorph) or: [m isKindOf: CompoundTileMorph]]! !


!URLMorph methodsFor: 'private' stamp: 'di 11/12/2000 15:44'!
enclosingPage
	"Answer the inner-most SqueakPage contents that contains this morph, or nil if there isn't one."

	self allOwnersSuchThat: [:m | m isKindOf: PasteUpMorph]
		do: [:m | (SqueakPageCache pageForMorph: m) ifNotNilDo: [:pg | ^ pg]].
	^ nil
! !

Morph removeSelector: #couldBeOwnedBy:!
