'From Squeak 2.2 of Sept 23, 1998 on 5 October 1998 at 6:07:25 pm'!
"Change Set:		loadFromDisk-sw
Date:			5 October 1998
Author:			Scott Wallace

Adds a single menu item to the New Morph menu which will present you with a list of all files loadable as morphs from disk, whether they be .morph files or .gif files or .jpg files, etc."!


!HandMorph methodsFor: 'world menu' stamp: 'sw 10/5/1998 18:04'!
newMorph
	| menu subMenu catDict shortCat class |
	menu _ (MenuMorph entitled: 'Add a new morph') defaultTarget: self.
	menu addStayUpItem.
	menu add: 'from paste buffer' action: #pasteMorph.
	menu add: 'from a file...' target: self action: #readMorphFromAFile.
	menu add: 'grab patch from screen' action: #grabDrawingFromScreen.
	menu add: 'make new drawing' target: self action: #newDrawingFromMenu.

	menu addLine.
	catDict _ Dictionary new.
	SystemOrganization categories do:
		[:cat |
		((cat beginsWith: 'Morphic-')
				and: [(#('Morphic-Menus' 'Morphic-Support') includes: cat) not])
		ifTrue:
			[shortCat _ cat copyFrom: 'Morphic-' size+1 to: cat size.
			(SystemOrganization listAtCategoryNamed: cat) do:
				[:cName | class _ Smalltalk at: cName.
				((class inheritsFrom: Morph)
					and: [class includeInNewMorphMenu])
					ifTrue:
					[(catDict includesKey: shortCat) 
					ifTrue: [(catDict at: shortCat) addLast: class]
					ifFalse: [catDict at: shortCat put: (OrderedCollection with: class)]]]]].

	catDict keys asSortedCollection do:
		[:categ |
		subMenu _ MenuMorph new.
		((catDict at: categ) asSortedCollection: [:c1 :c2 | c1 name < c2 name]) do:
			[:cl | subMenu add: cl name
					target: self
					selector: #newMorphOfClass:event:
					argument: cl].
		menu add: categ subMenu: subMenu].

	menu popUpAt: self position forHand: self.
! !

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 10/5/1998 17:59'!
readMorphFromAFile
	"Produce a morph from a file -- either a saved .morph file or a graphics file"

	| morphOrList ff aName f m |
	aName _ Utilities chooseFileWithSuffixFromList:  #('.morph' '.gif' '.bmp' '.jpg' '.form' ) withCaption: 'Choose a file
to load'.
	aName ifNil: [^ self].  "User made no choice"
	aName == #none ifTrue: [^ self inform: 
'Sorry, no suitable files found
(names must end in .morph, .gif,
.bmp, .jpeg, or .form)'].

	(aName asLowercase endsWith: '.morph')
		ifTrue:
			[ff _ FileStream oldFileNamed: aName.
			morphOrList _ ff fileInObjectAndCode.		"code filed in is the Model class"
			"the file may contain either a single morph or an array of morphs"
			self world addMorphsAndModel: morphOrList]
		ifFalse:
			[f _ Form fromFileNamed: aName.
			f ifNil: [^ self error: 'unrecognized image file format'].
			m _ self drawingClass new form: f.
			self attachMorph: m]
! !


!Utilities class methodsFor: 'summer97 additions' stamp: 'sw 10/5/1998 17:58'!
chooseFileWithSuffixFromList: aSuffixList withCaption: aCaption
	"Pop up a list of all files in the default directory which have a suffix in the list.  Return #none if there are none; return nil if the user backs out of the menu without making a choice."
	"Utilities chooseFileWithSuffixFromList: #('.gif' '.jpg')"
	| aList aName |
	aList _ OrderedCollection new.
	aSuffixList do:
		[:aSuffix | aList addAll: (FileDirectory default fileNamesMatching: '*', aSuffix)].
	^ aList size > 0
		ifTrue:
			[aName _ (SelectionMenu selections: aList) startUpWithCaption: aCaption.
			aName]
		ifFalse:
			[#none]! !


