'From Squeak3.1alpha of 28 February 2001 [latest update: #4237] on 6 August 2001 at 6:39:48 pm'!
"Change Set:		FileStreamContents
Date:			6 August 2001
Author:			Andreas Raab

Implement FileStream>>contents in favour of #allContentsWithoutSideEffects. Make good use of it for a hack in reading native resources (necessary because FileStream>>reset - called by ImageReadWriter - is implemented as #reopen which may not be possible if file resides outside Squeak sand box."!


!Form methodsFor: 'resources' stamp: 'ar 8/6/2001 18:36'!
readNativeResourceFrom: byteStream
	| img aStream |
	(byteStream isKindOf: FileStream) ifTrue:[
		"Ugly, but ImageReadWriter will send #reset which is implemented as #reopen and we may not be able to do so."
		aStream _ RWBinaryOrTextStream with: byteStream contents.
	] ifFalse:[
		aStream _ byteStream.
	].
	img _ [ImageReadWriter formFromStream: aStream] on: Error do:[:ex| nil].
	img ifNil:[^nil].
	"Forget color map if any"
	img _ Form extent: img extent depth: img depth bits: img bits.
	img displayInterpolatedOn: self.
	img _ nil.! !


!FileStream methodsFor: 'accessing' stamp: 'ar 8/6/2001 18:34'!
contents
	"Return the contents of the receiver. Do not close or otherwise touch the receiver. Return data in whatever mode the receiver is in (e.g., binary or text)."
	| s savePos |
	savePos _ self position.
	self position: 0.
	s _ self next: self size.
	self position: savePos.
	^s! !


!ProjectLoading class methodsFor: 'as yet unclassified' stamp: 'ar 8/6/2001 18:33'!
openName: aFileName stream: preStream fromDirectory: aDirectoryOrNil withProjectView: existingView
	"Reconstitute a Morph from the selected file, presumed to be represent a Morph saved via the SmartRefStream mechanism, and open it in an appropriate Morphic world."
 	| morphOrList proj trusted localDir projStream archive mgr projectsToBeDeleted |
	(preStream isNil or: [preStream size = 0]) ifTrue: [
		ProgressNotification  signal: '9999 about to enter project'.		"the hard part is over"
		^self inform: 
'It looks like a problem occurred while
getting this project. It may be temporary,
so you may want to try again,'
	].
	ProgressNotification signal: '2:fileSizeDetermined ',preStream size printString.
	preStream isZipArchive 
		ifTrue:[	archive _ ZipArchive new readFrom: preStream.
				projStream _ self projectStreamFromArchive: archive]
		ifFalse:[projStream _ preStream].
	trusted _ SecurityManager default positionToSecureContentsOf: projStream.
	trusted ifFalse:[(SecurityManager default enterRestrictedMode) ifFalse:[
		(preStream respondsTo: #close) ifTrue:[preStream close].
		^self]].

	localDir _ Project squeakletDirectory.
	aFileName ifNotNil: [
		(aDirectoryOrNil isNil or: [aDirectoryOrNil pathName ~= localDir pathName]) ifTrue: [
			localDir deleteFileNamed: aFileName.
			(localDir fileNamed: aFileName) 
				nextPutAll: preStream contents;
				close.
		].
	].
	morphOrList _ projStream asUnZippedStream.
	preStream sleep.		"if ftp, let the connection close"
	ProgressNotification  signal: '3:unzipped'.
	ResourceCollector current: ResourceCollector new.
	morphOrList _ morphOrList fileInObjectAndCode.
	mgr _ ResourceManager new initializeFrom: ResourceCollector current.
	mgr registerUnloadedResources.
	archive ifNotNil:[mgr preLoadFromArchive: archive cacheName: aFileName].
	(preStream respondsTo: #close) ifTrue:[preStream close].
	ResourceCollector current: nil.
	ProgressNotification  signal: '4:filedIn'.
	ProgressNotification  signal: '9999 about to enter project'.		"the hard part is over"
	(morphOrList isKindOf: ImageSegment) ifTrue: [
		proj _ morphOrList arrayOfRoots 
			detect: [:mm | mm class == Project] 
			ifNone: [^self inform: 'No project found in this file'].
		proj resourceManager: mgr.
		proj versionFrom: preStream.
		proj lastDirectory: aDirectoryOrNil.
		CurrentProjectRefactoring currentBeParentTo: proj.
		projectsToBeDeleted _ OrderedCollection new.
		existingView ifNil: [
			Smalltalk isMorphic ifTrue: [
				proj createViewIfAppropriate.
			] ifFalse: [
				ChangeSorter allChangeSets add: proj changeSet.
				ProjectView openAndEnter: proj.
				"Note: in MVC we get no further than the above"
			].
		] ifNotNil: [
			(existingView project isKindOf: DiskProxy) ifFalse: [
				existingView project changeSet name: ChangeSet defaultName.
				projectsToBeDeleted add: existingView project.
			].
			(existingView owner isKindOf: SystemWindow) ifTrue: [
				existingView owner model: proj
			].
			existingView project: proj.
		].
		ChangeSorter allChangeSets add: proj changeSet.
		Project current projectParameters 
			at: #deleteWhenEnteringNewProject 
			ifPresent: [ :ignored | 
				projectsToBeDeleted add: Project current.
				Project current removeParameter: #deleteWhenEnteringNewProject.
			].
		projectsToBeDeleted isEmpty ifFalse: [
			proj projectParameters 
				at: #projectsToBeDeleted 
				put: projectsToBeDeleted.
		].
		^ ProjectEntryNotification signal: proj
	].

	(morphOrList isKindOf: SqueakPage) ifTrue: [
		morphOrList _ morphOrList contentsMorph
	].
	(morphOrList isKindOf: PasteUpMorph) ifFalse: [
		^ self inform: 'This is not a PasteUpMorph or exported Project.'
	].
	(Project newMorphicOn: morphOrList) enter
! !

FileStream removeSelector: #allContentsWithoutSideEffects!
PositionableStream removeSelector: #allContentsWithoutSideEffects!
