'From Squeak3.2alpha of 2 October 2001 [latest update: #4525] on 21 November 2001 at 4:43 pm'!
"Change Set:		FlapsOnBottom
Date:			17 November 2001
Author:			Dan Ingalls

A number of tweaks to textual flapTabs.

1.  Adds the labelString as an instVar of FlapTab to eliminate arcane code that recovered it from the working string in the textmorph.

2.  Uses kerning to avoid the need to intersperse spaces in horizontal labels.  This also gives finer control over spacing.

3.  Reduces the spacing around the labels generally, while attempting a decent balance with border widths, in all four orientations.

4.  Introduces a checkbox to show the navigator on a SharedFlap.  If this is enabled, it will suppress the current interface, regardless of the setting of #showProjectNavigator.  This is reflected in the help strings for the project navigator.

5.  Introduces a command, sharedFlapsAlongBottom, in the 'flaps' menu, that puts all shared tabs along the bottom right of the screen.  This typically reclaims a fair amount of screen area, reduces interference with pop-out scroll bars and collapsed morphs, and also requires less computation to redisplay flap tabs.  Window placement algorithms in RealEstateAgent have been revised to take advantage of this.

6.  Adds 'Stack Tools' and 'Navigator' to the set of default shared flaps.

7.  (under the hood) Eliminated a bunch of methods for locating or replacing specific tabs, in favor of a single method paramterized by ID.  Also introduced a mechanism for tolerating assigned IDs with sequence numbers."!

ReferenceMorph subclass: #FlapTab
	instanceVariableNames: 'flapShowing edgeToAdhereTo slidesOtherObjects popOutOnDragOver popOutOnMouseOver inboard dragged lastReferentThickness edgeFraction labelString '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Flaps'!
RectangleMorph subclass: #MorphExample
	instanceVariableNames: 'phase ball star '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Demo'!

!Flaps class methodsFor: 'flap mechanics' stamp: 'di 11/19/2001 23:38'!
reinstateDefaultFlaps
	"Remove all existing 'standard' global flaps clear the global list, and and add fresh ones.  To be called by doits in updates etc.  This is a radical step, but it does *not* clobber non-standard global flaps or local flaps.  To get the effect of the *former* version of this method, call Flaps freshFlapsStart"

	"Flaps reinstateDefaultFlaps"
	self globalFlapTabsIfAny do:
		[:aFlapTab |
			(#('Painting' 'Stack Tools' 'Squeak' 'Menu' 'Widgets' 'Tools' 'Supplies' 'Scripting' 'Objects' 'Navigator') includes: aFlapTab flapID) ifTrue:
				[self removeFlapTab: aFlapTab keepInList: false]].

	"The following reduces the risk that flaps will be created with variant IDs
		such as 'Stack Tools2', potentially causing some shared flap logic to fail."
		"Smalltalk garbageCollect."  "-- see if we are OK without this"

	self addStandardFlaps.
	"self disableGlobalFlapWithID: 'Scripting'.
	self disableGlobalFlapWithID: 'Objects'."
	self currentWorld addGlobalFlaps.
	self currentWorld reformulateUpdatingMenus.
! !

!Flaps class methodsFor: 'shared flaps' stamp: 'di 11/19/2001 22:07'!
globalFlapTabWithID: aFlapID
	"answer the global flap tab with the given id, or nil if none"

	^ self globalFlapTabsIfAny detect: [:aFlapTab | aFlapTab flapID = aFlapID]
		ifNone:
		["Second try allows sequence numbers"
		self globalFlapTabsIfAny detect: [:aFlapTab | FlapTab givenID: aFlapTab flapID matches: aFlapID]
			ifNone: [nil]]! !

!Flaps class methodsFor: 'shared flaps' stamp: 'di 11/21/2001 15:10'!
sharedFlapsAlongBottom
	"Flaps sharedFlapsAlongBottom"

	| leftX |
	leftX _ Display width-15.

	"Pace off in order from right to left, setting positions"
	#('Squeak' 'Navigator' 'Supplies' 'Widgets' 'Stack Tools' 'Tools') reverseDo:
		[:id | (self globalFlapTabWithID: id) ifNotNilDo:
			[:ft | ft setEdge: #bottom.
			ft right: leftX - 3.  leftX _ ft left]].

	"Put Nav Bar centered under tab if possible"
	(self globalFlapTabWithID: 'Navigator') ifNotNilDo:
		[:ft | ft referent left: (ft center x - (ft referent width//2) max: 0)].
! !

!Flaps class methodsFor: 'menu commands' stamp: 'di 11/19/2001 23:08'!
disableGlobalFlaps: interactive
	"Clobber all the shared flaps structures.  First read the user her Miranda rights."

	interactive
		ifTrue: [(self confirm: 
'CAUTION!! This will destroy all the shared
flaps, so that they will not be present in 
*any* project.  If, later, you want them
back, you will have to reenable them, from
this same menu, whereupon the standard
default set of shared flaps will be created.
Do you really want to go ahead and clobber
all shared flaps at this time?') ifFalse: [^ self]].

	self globalFlapTabsIfAny do:
		[:aFlapTab | self removeFlapTab: aFlapTab keepInList: false.
		aFlapTab isInWorld ifTrue: [self error: 'Flap problem']].
	self clobberFlapTabList.
	SharedFlapsAllowed _ false.
	Smalltalk isMorphic ifTrue:
		[ActiveWorld restoreMorphicDisplay.
		ActiveWorld reformulateUpdatingMenus].

	"The following reduces the risk that flaps will be created with variant IDs
		such as 'Stack Tools2', potentially causing some shared flap logic to fail."
		"Smalltalk garbageCollect."  "-- see if we are OK without this"
! !

!Flaps class methodsFor: 'menu commands' stamp: 'di 11/18/2001 13:33'!
enableDisableGlobalFlapWithID: aFlapID
	"Toggle the enable/disable status of the given global flap"

	| disabledFlapIDs  aFlapTab currentProject |
	(currentProject _ Project current) assureFlapIntegrity.
	Smalltalk isMorphic ifFalse: [^ self].
	disabledFlapIDs _ currentProject parameterAt: #disabledGlobalFlapIDs.
	(aFlapTab _ self globalFlapTabWithID: aFlapID) ifNotNil:
		[aFlapTab hideFlap].
	(disabledFlapIDs includes: aFlapID)
		ifTrue:
			[disabledFlapIDs remove: aFlapID.
			self currentWorld addGlobalFlaps]
		ifFalse:
			[disabledFlapIDs add: aFlapID.
			aFlapTab ifNotNil: [aFlapTab delete]].
	aFlapID = 'Navigator' ifTrue: [Project current assureNavigatorPresenceMatchesPreference]
	! !

!Flaps class methodsFor: 'new flap' stamp: 'di 11/19/2001 21:07'!
newFlapTitled: aString onEdge: anEdge inPasteUp: aPasteUpMorph
	"Add a flap with the given title, placing it on the given edge, in the given pasteup"

	| aFlapBody aFlapTab  |
	aFlapBody _ PasteUpMorph newSticky.
	aFlapTab _ FlapTab new referent: aFlapBody.
	aFlapTab setName: aString edge: anEdge color: (Color r: 0.516 g: 0.452 b: 1.0).

	anEdge == #left ifTrue:
		[aFlapTab position: (aPasteUpMorph left @ aPasteUpMorph top).
		aFlapBody extent: (200 @ aPasteUpMorph height)].
	anEdge == #right ifTrue:
		[aFlapTab position: ((aPasteUpMorph right - aFlapTab width) @ aPasteUpMorph top).
		aFlapBody extent: (200 @ aPasteUpMorph height)].
	anEdge == #top ifTrue:
		[aFlapTab position: ((aPasteUpMorph left + 50) @ aPasteUpMorph top).
		aFlapBody extent: (aPasteUpMorph width @ 200)].
	anEdge == #bottom ifTrue:
		[aFlapTab position: ((aPasteUpMorph left + 50) @ (aPasteUpMorph bottom - aFlapTab height)).
		aFlapBody extent: (aPasteUpMorph width @ 200)].

	aFlapBody beFlap: true.
	aFlapBody color: self defaultColorForFlapBackgrounds.

	^ aFlapTab! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 20:59'!
addAndEnableEToyFlaps
	"Initialize the standard default out-of-box set of global flaps.  This method creates them and places them in my class variable #SharedFlapTabs, but does not itself get them displayed."

	SharedFlapTabs ifNil: [SharedFlapTabs _ OrderedCollection new].

	SharedFlapTabs add: self newSuppliesFlap.
	SharedFlapTabs add: self newWidgetsFlap.
	SharedFlapTabs add: self newNavigatorFlap.

	self enableGlobalFlapWithID: 'Supplies'.
	self enableGlobalFlapWithID: 'Navigator'.
	self enableGlobalFlapWithID: 'Widgets'.

	SharedFlapsAllowed _ true.
	Project current flapsSuppressed: false.
	^ SharedFlapTabs

"Flaps addAndEnableEToyFlaps"! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/21/2001 16:33'!
addNewDefaultSharedFlaps
	"Add the stack tools flap and the navigator flap to the global list, but do not have them showing initially.  Transitional, called by the postscript of the FlapsOnBottom update; probably dispensable afterwards."

	SharedFlapTabs ifNotNil:
		[(self globalFlapTabWithID: 'Stack Tools') ifNil:
			[SharedFlapTabs add: self newStackToolsFlap delete].
		self enableGlobalFlapWithID: 'Stack Tools'.
		(self globalFlapTabWithID: 'Navigator') ifNil:
			[SharedFlapTabs add: self newNavigatorFlap delete].
		self enableGlobalFlapWithID: 'Navigator'.
		self currentWorld addGlobalFlaps]! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/21/2001 14:52'!
addStandardFlaps
	"Initialize the standard default out-of-box set of global flaps.  This method creates them and places them in my class variable #SharedFlapTabs, but does not itself get them displayed."

	SharedFlapTabs ifNil: [SharedFlapTabs _ OrderedCollection new].

	SharedFlapTabs add: self newSqueakFlap.
	SharedFlapTabs add: self newSuppliesFlap.
	SharedFlapTabs add: self newToolsFlap.
	SharedFlapTabs add: self newWidgetsFlap.
	SharedFlapTabs add: self newStackToolsFlap.
	SharedFlapTabs add: self newNavigatorFlap.
	SharedFlapTabs add: self newPaintingFlap.
	"SharedFlapTabs add: self newScriptingFlap delete.
	SharedFlapTabs add: self newObjectsFlap."

	self enableGlobalFlapWithID: 'Squeak'.	
	self enableGlobalFlapWithID: 'Supplies'.
	self enableGlobalFlapWithID: 'Tools'.
	self enableGlobalFlapWithID: 'Controls'.
	self enableGlobalFlapWithID: 'Stack Tools'.
	self disableGlobalFlapWithID: 'Painting'.
	"self disableGlobalFlapWithID: 'Scripting'.
	self enableGlobalFlapWithID: 'Objects'."

	^ SharedFlapTabs

"Flaps reinstateDefaultFlaps"! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 21:09'!
newLoneSuppliesFlap
	"Answer a fully-instantiated flap named 'Supplies' to be placed at the bottom of the screen, for use when it is the only flap shown upon web launch"

	|  aFlapTab aStrip leftEdge |  "Flaps setUpSuppliesFlapOnly"
	aStrip _ PartsBin newPartsBinWithOrientation: #leftToRight from:	 #(

	(TrashCanMorph			new						'Trash'				'A tool for discarding objects')	
	(ScriptingSystem 		scriptControlButtons 			'Status'				'Buttons to run, stop, or single-step scripts')
	(AllScriptsTool			allScriptsToolForActiveWorld	'All Scripts' 		'A tool that lets you control all the running scripts in your world')

	(PaintInvokingMorph	new						'Paint'				'Drop this into an area to start making a fresh painting there')
	(RectangleMorph 		authoringPrototype		'Rectangle' 		'A rectangle'	)
	(RectangleMorph		roundRectPrototype		'RoundRect'		'A rectangle with rounded corners')
	(EllipseMorph			authoringPrototype		'Ellipse'			'An ellipse or circle')
	(StarMorph				authoringPrototype		'Star'			'A star')
	(CurveMorph			authoringPrototype		'Curve'			'A curve')
	(PolygonMorph			authoringPrototype		'Polygon'		'A straight-sided figure with any number of sides')
	(TextMorph				authoringPrototype		'Text'			'Text that you can edit into anything you desire.')
	(SimpleSliderMorph		authoringPrototype		'Slider'			'A slider for showing and setting numeric values.')
	(JoystickMorph			authoringPrototype		'Joystick'		'A joystick-like control')
	(ScriptingSystem		prototypicalHolder 		'Holder'			'A place for storing alternative pictures in an animation, ec.')
	(ScriptableButton		authoringPrototype		'Button'			'A Scriptable button')
	(PasteUpMorph			authoringPrototype		'Playfield'		'A place for assembling parts or for staging animations')
	(BookMorph				authoringPrototype		'Book'			'A multi-paged structure')
	(TabbedPalette			authoringPrototype		'Tabs'			'A structure with tabs')

	(RecordingControlsMorph authoringPrototype			'Sound'				'A device for making sound recordings.')
	(MagnifierMorph		newRound					'Magnifier'			'A magnifying glass')

	(ImageMorph			authoringPrototype		'Picture'		'A non-editable picture of something')
	(ClockMorph				authoringPrototype		'Clock'			'A simple digital clock')
	(BookMorph				previousPageButton 		'Previous'		'A button that takes you to the previous page')
	(BookMorph				nextPageButton			'Next'			'A button that takes you to the next page')
		).

	aFlapTab _ FlapTab new referent: aStrip beSticky.
	aFlapTab setName: 'Supplies' edge: #bottom color: Color red lighter.

	aStrip extent: self currentWorld width @ 78.
	leftEdge _ ((Display width - (16  + aFlapTab width)) + 556) // 2.

	aFlapTab position: (leftEdge @ (self currentWorld height - aFlapTab height)).

	aStrip beFlap: true.
	aStrip color: Color red muchLighter.
	aStrip autoLineLayout: true.
	
	^ aFlapTab! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:45'!
newNavigatorFlap
	"Answer a newly-created flap which adheres to the bottom edge of the screen and which holds the project navigator controls. "

	|  aFlapTab navBar aFlap |
	navBar _ ProjectNavigationMorph preferredNavigator new.
	aFlap _ PasteUpMorph newSticky borderWidth: 0;
			extent: navBar extent + (0@20);
			color: (Color orange alpha: 0.8);
			beFlap: true;
			addMorph: navBar.
	navBar fullBounds.  "to establish width"
	
	aFlapTab _ FlapTab new referent: aFlap.
	aFlapTab setName: 'Navigator' edge: #bottom color: Color orange.
	aFlapTab position: ((navBar width // 2) - (aFlapTab width // 2))
					@ (self currentWorld height - aFlapTab height).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.
	^ aFlapTab

"Flaps replaceGlobalFlapwithID: 'Navigator' "
! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 21:22'!
newObjectsFlap
	"Answer a fully-instantiated flap named 'Objects' to be placed at the top of the screen.  Not currently called; this worked once, but probably not at the moment."

	|  aFlapTab anObjectsTool |
	anObjectsTool _ ObjectsTool new.
	anObjectsTool initializeForFlap.
	anObjectsTool showCategories.

	aFlapTab _ FlapTab new referent: anObjectsTool beSticky.
	aFlapTab setName: 'Objects' edge: #top color: Color red lighter.
	aFlapTab position: ((Display width - (aFlapTab width + 22)) @ 0).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	anObjectsTool extent: self currentWorld width @ 200.
	anObjectsTool beFlap: true.
	anObjectsTool color: Color red muchLighter.
	anObjectsTool clipSubmorphs: true.
	
	^ aFlapTab! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:27'!
newScriptingFlap
	"Add a flap with scriptors' tools in it"

	| aFlapTab aStrip |
	aStrip _ PartsBin newPartsBinWithOrientation: #topToBottom from: self quadsDeiningScriptingFlap.
	aFlapTab _ FlapTab new referent: aStrip beSticky.
	aFlapTab setName: 'Scripting' edge: #left color: (Color r: 0.548 g: 1.0 b: 0.935).
	aFlapTab position: (0 @ 80).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	aStrip beFlap: true.
	aStrip extent: 120 @ self currentWorld height.
	aStrip color: (aFlapTab color lighter lighter alpha: 0.85).
	^ aFlapTab! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:26'!
newSqueakFlap
	"Answer a new default 'Squeak' flap for the left edge of the screen"

	| aFlap aFlapTab aButton aClock buttonColor anOffset bb |
	aFlap _ PasteUpMorph newSticky borderWidth: 0.
	aFlapTab _ FlapTab new referent: aFlap.
	aFlapTab setName: 'Squeak' edge: #left color: Color brown lighter lighter.
	aFlapTab position: (0 @ ((Display height - aFlapTab height) // 2)).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	aFlap cellInset: 14@14.
	aFlap beFlap: true.
	aFlap color: (Color brown muchLighter lighter "alpha: 0.3").
	aFlap extent: 150 @ self currentWorld height.
	aFlap layoutPolicy: TableLayout new.
	aFlap wrapCentering: #topLeft.
	aFlap layoutInset: 2.
	aFlap listDirection: #topToBottom.
	aFlap wrapDirection: #leftToRight.

	self addProjectNavigationButtonsTo: aFlap.
	anOffset _ 16.

	buttonColor _ Color green muchLighter.
	bb _ SimpleButtonMorph new target: Smalltalk.
	bb color: buttonColor.
	aButton _ bb copy.
	aButton actionSelector: #saveSession.
	aButton setBalloonText: 'Make a complete snapshot of the current state of the image onto disk.'.
	aButton label: 'snapshot'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ bb copy target: Utilities.
	aButton actionSelector: #fileOutChanges.
	aButton label: 'file out changes'.
	aButton setBalloonText: 'File out the current change set to disk.'.
	aFlap addMorph: aButton.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ bb copy target: Utilities.
	aButton actionSelector: #openRecentSubmissionsBrowser.
	aButton setBalloonText: 'Open a message-list browser showing the most-recently-submitted methods.'.
	aButton label: 'recent submissions'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aClock _ ClockMorph newSticky.
	aClock color: Color red.
	aClock showSeconds: false.
	aClock font: (TextStyle default fontAt: 3).
	aClock step.
	aClock setBalloonText: 'The time of day.  If you prefer to see seconds, check out my menu.'.
	aFlap addCenteredAtBottom: aClock offset: anOffset.

	aButton _ bb copy target: Preferences.
	aButton actionSelector: #openPreferencesInspector.
	aButton setBalloonText: 'Open a window allowing me to view and change various Preferences.'.
	aButton label: 'preferences...'.
	aButton color: Color cyan muchLighter.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	aButton _ bb copy target: Utilities.
	aButton actionSelector: #updateFromServer.
	aButton label: 'load code updates'.
	aButton color: Color cyan muchLighter.
	aButton setBalloonText: 'Check the Squeak server for any new code updates, and load any that are found.'.
	aFlap addCenteredAtBottom: aButton offset: anOffset.

	self addSystemStatusLinesTo: aFlap.

	aButton _ TrashCanMorph newSticky.
	aFlap addCenteredAtBottom: aButton offset: anOffset.
	aButton startStepping.

	^ aFlapTab

"Flaps replaceGlobalFlapwithID: 'Squeak' "! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:26'!
newStackToolsFlap
	"Add a flap with stack tools in it"

	| aFlapTab aStrip |
	aStrip _ PartsBin newPartsBinWithOrientation: #leftToRight from:	 self quadsDefiningStackToolsFlap.

	aFlapTab _ FlapTab new referent: aStrip beSticky.
	aFlapTab setName: 'Stack Tools' edge: #bottom color: Color brown lighter lighter.
	aFlapTab position: ((Display width - (aFlapTab width + 226)) @ (self currentWorld height - aFlapTab height)).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	aStrip extent: self currentWorld width @ 78.
	aStrip beFlap: true.
	aStrip autoLineLayout: true.
	aStrip color: (Color red muchLighter "alpha: 0.2").
	aStrip extent: self currentWorld width @ 70.

	^ aFlapTab

"Flaps replaceGlobalFlapwithID: 'Stack Tools' "! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:26'!
newSuppliesFlap
	"Answer a fully-instantiated flap named 'Supplies' to be placed at the bottom of the screen"

	|  aFlapTab aStrip |
	aStrip _ PartsBin newPartsBinWithOrientation: #leftToRight from:	 self quadsDefiningSuppliesFlap.

	aFlapTab _ FlapTab new referent: aStrip beSticky.
	aFlapTab setName: 'Supplies' edge: #bottom color: Color red lighter.
	aFlapTab position: ((Display width - (aFlapTab width + 22)) @ (self currentWorld height - aFlapTab height)).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	aStrip extent: self currentWorld width @ 78.
	aStrip beFlap: true.
	aStrip color: Color red muchLighter.
	aStrip autoLineLayout: true.
	
	^ aFlapTab

"Flaps replaceGlobalFlapwithID: 'Supplies' "! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:26'!
newToolsFlap
	"Answer a newly-created flap which adheres to the right edge of the screen and which holds prototypes of standard tools."

	|  aFlapTab aStrip |
	aStrip _ PartsBin newPartsBinWithOrientation: #topToBottom from: self quadsDefiningToolsFlap.
 
	aFlapTab _ FlapTab new referent: aStrip beSticky.
	aFlapTab setName: 'Tools' edge: #right color: Color orange lighter.
	aFlapTab position: (self currentWorld width - aFlapTab width) @ ((Display height - aFlapTab height) // 2).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	aStrip extent: (90 @ self currentWorld height).
	aStrip beFlap: true.
	aStrip color: (Color orange muchLighter alpha: 0.8).
	
	^ aFlapTab

"Flaps replaceGlobalFlapwithID: 'Tools' "
! !

!Flaps class methodsFor: 'predefined flaps' stamp: 'di 11/19/2001 22:26'!
newWidgetsFlap
	"Answer a newly-created flap which adheres to the bottom edge of the screen and which holds prototypes of standard widgets. "

	|  aFlapTab aStrip |
	aStrip _ PartsBin newPartsBinWithOrientation: #leftToRight from:	 self quadsDefiningWidgetsFlap.

	aFlapTab _ FlapTab new referent: aStrip beSticky.
	aFlapTab setName: 'Widgets' edge: #bottom color: Color blue lighter lighter.
	aFlapTab position: ((Display width - (aFlapTab width + 122)) @ (self currentWorld height - aFlapTab height)).
	aFlapTab setBalloonText: aFlapTab balloonTextForFlapsMenu.

	aStrip extent: self currentWorld width @ 78.
	aStrip beFlap: true.
	aStrip color: (Color blue muchLighter alpha: 0.8).
	aStrip autoLineLayout: true.
	
	^ aFlapTab

"Flaps replaceGlobalFlapwithID: 'Widgets' "
! !

!Flaps class methodsFor: 'replacement' stamp: 'di 11/19/2001 22:21'!
replaceGlobalFlapwithID: flapID
	"If there is a global flap with flapID, replace it with an updated one."

	| aFlapTab replacement |
	(aFlapTab _ self globalFlapTabWithID: flapID) ifNil: [^ self].
	self removeFlapTab: aFlapTab keepInList: false.
	flapID = 'Stack Tools' ifTrue: [replacement _ self newStackToolsFlap].
	flapID = 'Supplies' ifTrue: [replacement _ self newSuppliesFlap].
	flapID = 'Tools' ifTrue: [replacement _ self newToolsFlap].
	flapID = 'Widgets' ifTrue: [replacement _ self newWidgetsFlap].
	flapID = 'Navigator' ifTrue: [replacement _ self newNavigatorFlap].
	flapID = 'Squeak' ifTrue: [replacement _ self newSqueakFlap].
	replacement ifNil: [^ self].
	self addGlobalFlap: replacement.
	self currentWorld ifNotNil: [self currentWorld addGlobalFlaps]

"Flaps replaceFlapwithID: 'Widgets' "! !

!Flaps class methodsFor: 'replacement' stamp: 'di 11/19/2001 22:20'!
replaceToolsFlap
	"if there is a global tools flap, replace it with an updated one."

	self replaceGlobalFlapwithID: 'Tools' 

"Flaps replaceToolsFlap"! !


!Morph methodsFor: 'menus' stamp: 'di 11/21/2001 09:44'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	| realOwner realMorph |
	"Add morph-specific items to the given menu which was invoked by the given hand.  Note the special-casing of Worlds, for which some of the the otherwise generic items are excluded."
	aCustomMenu addUpdating: #hasDragAndDropEnabledString action: #changeDragAndDrop.

	self isWorldMorph
		ifFalse:
			[(self isKindOf: SystemWindow)
				ifFalse: [aCustomMenu add: 'put in a window' action: #embedInWindow].
			aCustomMenu addUpdating: #stickinessString target: self action: #toggleStickiness.
			aCustomMenu add: 'adhere to edge...' action: #adhereToEdge]
		ifTrue:
			[Flaps sharedFlapsAllowed ifTrue:
				[aCustomMenu addUpdating: #suppressFlapsString
						target: CurrentProjectRefactoring 
						action: #currentToggleFlapsSuppressed].
			aCustomMenu add: 'desktop menu...' target: self action: #putUpDesktopMenu:.
			aCustomMenu addLine].

	Preferences noviceMode ifFalse:
		[self addDebuggingItemsTo: aCustomMenu hand: aHandMorph].

	realOwner _ (realMorph _ self topRendererOrSelf) owner.
	(realOwner isKindOf: TextPlusPasteUpMorph) ifTrue:
		[aCustomMenu add: 'GeeMail stuff...' subMenu: (realOwner textPlusMenuFor: realMorph)].

	aCustomMenu addLine.
	(self isStackBackground) ifFalse: [
		aCustomMenu add: 'be a card in an existing stack...' action: #insertAsStackBackground].
	aCustomMenu add: 'make an instance for my data' action: #abstractAModel.
	(self isStackBackground) ifFalse: [
		aCustomMenu add: 'become a stack of cards' action: #wrapWithAStack].
	aCustomMenu addLine.
! !


!FlapTab methodsFor: 'initialization' stamp: 'di 11/18/2001 13:09'!
provideDefaultFlapIDBasedOn: aStem
	"Provide the receiver with a default flap id"

	| aNumber usedIDs anID  |
	aNumber _ 0.
	usedIDs _ FlapTab allSubInstances select: [:f | f ~~ self] thenCollect: [:f | f flapIDOrNil].
	anID _ aStem.
	[usedIDs includes: anID] whileTrue:
		[aNumber _ aNumber + 1.
		anID _ aStem, (aNumber asString)].
	self flapID: anID.
	^ anID! !

!FlapTab methodsFor: 'initialization' stamp: 'di 11/19/2001 21:20'!
setName: nameString edge: edgeSymbol color: flapColor
	"Set me up with the usual..."

	self setNameTo: nameString.
	self edgeToAdhereTo: edgeSymbol; inboard: false.
	self assumeString: nameString font: Preferences standardFlapFont
		orientation: self orientation color: flapColor.
	self setToPopOutOnDragOver: true.
	self setToPopOutOnMouseOver: false.
! !

!FlapTab methodsFor: 'initialization' stamp: 'di 11/21/2001 15:06'!
updateOldInstances
	| longForm |
	self isCurrentlyTextual ifTrue:
		["Old code -- delete after updating old instances."
		longForm _ submorphs first contents.
		labelString _ self orientation == #vertical
			ifTrue:
				[longForm asString copyWithout: Character cr]
			ifFalse:
				[(longForm asString collectWithIndex:
					[:ch :i | i even ifFalse: [$] ifTrue: [ch]]) copyWithout: $].
		self reformatTextualTab]! !

!FlapTab methodsFor: 'edge' stamp: 'di 11/20/2001 07:44'!
setEdge: anEdge
	"Set the edge as indicated, if possible"

	| newOrientation |
	self edgeToAdhereTo = anEdge ifTrue: [^ self].
	newOrientation _ nil.
	self orientation == #vertical
		ifTrue: [(#(top bottom) includes: anEdge) ifTrue:
					[newOrientation _ #horizontal]]
		ifFalse: [(#(top bottom) includes: anEdge) ifFalse:
					[newOrientation _ #vertical]].
	self edgeToAdhereTo: anEdge.
	newOrientation ifNotNil: [self transposeParts].
	referent isInWorld ifTrue: [self positionReferent].
	self adjustPositionVisAVisFlap! !

!FlapTab methodsFor: 'positioning' stamp: 'di 11/19/2001 12:19'!
fitContents
	self isCurrentlyTextual ifFalse: [^ super fitContents].
	self ifVertical:
		[self extent: submorphs first extent + (2 * self borderWidth) + (0@4).
		submorphs first position: self position + self borderWidth + (1@4)]
	ifHorizontal:
		[self extent: submorphs first extent + (2 * self borderWidth) + (8@-1).
		submorphs first position: self position + self borderWidth + (5@1)]! !

!FlapTab methodsFor: 'positioning' stamp: 'di 11/21/2001 16:02'!
transposeParts
	"The receiver's orientation has just been changed from vertical to horizontal or vice-versa."
	"First expand the flap to screen size, letting the submorphs lay out to fit,
	and then shrink the minor dimension back to the last row."

	self isCurrentlyTextual ifTrue:  "First recreate the tab with proper orientation"
		[self assumeString: self existingWording font: Preferences standardFlapFont
			orientation: self orientation color: self color].
	self orientation == #vertical
		ifTrue:	"changed from horizontal"
			[referent listDirection: #topToBottom; wrapDirection: #leftToRight.
			referent hasSubmorphs ifTrue:
				[referent extent: self currentWorld extent.
				referent fullBounds.  "Needed to trigger layout"
				referent width: (referent submorphs collect: [:m | m right]) max
									- referent left + self width]]
		ifFalse:
			[referent listDirection: #leftToRight; wrapDirection: #topToBottom.
			referent hasSubmorphs ifTrue:
				[referent extent: self currentWorld extent.
				referent fullBounds.  "Needed to trigger layout"
				referent height: (referent submorphs collect: [:m | m bottom]) max
									- referent top + self height]].
	referent hasSubmorphs ifFalse: [referent extent: 100@100].

	self spanWorld.
	flapShowing ifTrue: [self showFlap]! !

!FlapTab methodsFor: 'textual tabs' stamp: 'di 11/19/2001 22:47'!
assumeString: aString font: aFont orientation: orientationSymbol color: aColor
	| aTextMorph workString tabStyle |
	labelString _ aString asString.
	(orientationSymbol == #vertical)
		ifTrue: [workString _ String streamContents:
					[:s | labelString do: [:c | s nextPut: c]
							separatedBy: [s nextPut: Character cr]]]
		ifFalse: [workString _ labelString].

	tabStyle _ TextStyle new leading: -4; newFontArray: (Array with: aFont).
	aTextMorph _ (TextMorph new setTextStyle: tabStyle)
					contents: (workString asText addAttribute: (TextKern kern: 3)).
	self removeAllMorphs.
	self borderWidth: 2; borderColor: #raised.
	aColor ifNotNil: [self color: aColor].
	self addMorph: aTextMorph centered.
	aTextMorph lock.
"
FlapTab allSubInstancesDo: [:ft | ft reformatTextualTab]
"
! !

!FlapTab methodsFor: 'textual tabs' stamp: 'di 11/17/2001 20:17'!
existingWording
	^ labelString! !

!FlapTab methodsFor: 'textual tabs' stamp: 'di 11/17/2001 20:22'!
useStringTab: aString
	| aLabel |
	labelString _ aString asString.
	aLabel _ StringMorph  new contents: labelString.
	self addMorph: aLabel.
	aLabel position: self position.
	aLabel highlightColor: self highlightColor; regularColor: self regularColor.
	aLabel lock.
	self fitContents.
	self layoutChanged! !

!FlapTab methodsFor: 'rounding' stamp: 'di 11/20/2001 08:20'!
roundedCorners
	edgeToAdhereTo == #bottom ifTrue: [^ #(1 4)].
	edgeToAdhereTo == #right ifTrue: [^ #(1 2)].
	edgeToAdhereTo == #left ifTrue: [^ #(3 4)].
	^ #(2 3)  "#top and undefined"
! !

!FlapTab methodsFor: 'miscellaneous' stamp: 'di 11/19/2001 22:01'!
balloonTextForFlapsMenu
	"Answer the balloon text to show on a menu item in the flaps menu that governs the visibility of the receiver in the current project"

	| id |
	id _ self flapID.
	#(
	('Squeak'		'A flap with various generally-useful controls for updating the system, navigating among projects, etc.')
	('Tools'			'A quick way to get browsers, change sorters, file lists, etc.')
	('Widgets'		'A variety of controls and media tools')
	('Supplies' 		'A source for many basic types of objects')
	('Stack Tools' 	'Tools for building stacks.  Caution!!  Powerful but young and underdocumented')
	('Scripting'		'Tools useful when doing tile scripting')
	('Navigator'		'Project navigator:  includes controls for navigating through linked projects.  Also supports finding, loading and publishing projects in a shared environment')
	('Painting'		'A flap housing the paint palette.  Click on the closed tab to make make a new painting')) do:
		[:pair | (FlapTab givenID: id matches: pair first) ifTrue: [^ pair second]].

	^ self balloonText! !


!FlapTab class methodsFor: 'as yet unclassified' stamp: 'di 11/19/2001 21:59'!
givenID: aFlapID matches: pureID
	"eg, FlapTab givenID: 'Stack Tools2' matches: 'Stack Tools' "

	^ aFlapID = pureID or:
		[(aFlapID beginsWith: pureID)
			and: [(aFlapID copyFrom: pureID size+1 to: aFlapID size)
					allSatisfy: [:c | c isDigit]]]! !


!MorphExample methodsFor: 'as yet unclassified' stamp: 'di 11/20/2001 21:27'!
initialize
	phase _ 1.
	self extent: 200@200.
	ball _ EllipseMorph new extent: 30@30.
	self addMorph: ((star _ StarMorph new extent: 150@150) center: self center)! !

!MorphExample methodsFor: 'as yet unclassified' stamp: 'di 11/20/2001 21:31'!
step
	phase _ phase\\8 + 1.
	phase = 1 ifTrue: [^ ball delete].
	phase = 4 ifTrue: [self addMorph: ball].
	ball align: ball center with: (star vertices at: (phase-3*2)).! !


!PasteUpMorph methodsFor: 'menu & halo' stamp: 'di 11/19/2001 10:06'!
reformulateUpdatingMenus
	"Give any updating menu morphs in the receiver a fresh kiss of life"

	(self submorphs select: [:m | m isKindOf: UpdatingMenuMorph]) do:
		[:m | m updateMenu].
	"NB: to do the perfect job here one might well want to extend across allMorphs here, but the expense upon project entry is seemingly too high a price to pay at this point"

	Project current assureNavigatorPresenceMatchesPreference! !


!Project methodsFor: 'menu messages' stamp: 'di 11/18/2001 14:50'!
assureNavigatorPresenceMatchesPreference
	"Make sure that the current project conforms to the presence/absence of the navigator"

	| navigator navType wantIt |
	Smalltalk isMorphic ifFalse: [^ self].
	wantIt _ Preferences showProjectNavigator and:
		[Flaps sharedFlapsAllowed not
		or: [self flapsSuppressed
		or: [(Project current isFlapIDEnabled: 'Navigator') not]]].
	navType _ ProjectNavigationMorph preferredNavigator.
	navigator _ world findA: navType.
	wantIt
		ifFalse:
			[navigator ifNotNil: [navigator delete]]
		ifTrue:
			[navigator isNil ifTrue: 
				[(navigator _ navType new)
					bottomLeft: world bottomLeft;
					openInWorld: world]]! !

!Project methodsFor: 'flaps support' stamp: 'di 11/18/2001 14:34'!
flapsSuppressed: aBoolean
	"Make the setting of the flag that governs whether global flaps are suppressed in the project be as indicated and add or remove the actual flaps"

	self projectPreferenceFlagDictionary at: #showSharedFlaps put: aBoolean not.
	self == Project current  "Typical case"
		ifTrue:
			[Preferences setPreference: #showSharedFlaps toValue: aBoolean not]
		ifFalse:   "Anomalous case where this project is not the current one."
			[aBoolean
				ifTrue:		
					[Flaps globalFlapTabsIfAny do:
						[:aFlapTab | Flaps removeFlapTab: aFlapTab keepInList: true]]

				ifFalse:
					[Smalltalk isMorphic  ifTrue:
						[self currentWorld addGlobalFlaps]]].
	Project current assureNavigatorPresenceMatchesPreference! !


!ProjectNavigationMorph methodsFor: 'as yet unclassified' stamp: 'RAA 6/29/2000 11:11'!
handlesMouseOver: evt

	^true! !


!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'di 11/20/2001 00:16'!
assignCollapseFrameFor: aSSView 
	"Offer up a location along the left edge of the screen for a collapsed SSView. Make sure it doesn't overlap any other collapsed frames."

	| grid otherFrames topLeft viewBox collapsedFrame extent newFrame verticalBorderDistance top |
	grid _ 8.
	verticalBorderDistance _ 8.
	aSSView isMorph
		ifTrue: [otherFrames _ (SystemWindow windowsIn: aSSView world satisfying: [:w | w ~= aSSView])
						collect: [:w | w collapsedFrame]
						thenSelect: [:rect | rect notNil].
				viewBox _ self reduceByFlaps: aSSView world viewBox]
		ifFalse: [otherFrames _ ScheduledControllers scheduledWindowControllers
						collect: [:aController | aController view ~= aSSView ifTrue: [aController view collapsedFrame]]
						thenSelect: [:rect | rect notNil].
				viewBox _ Display boundingBox].
	collapsedFrame _ aSSView collapsedFrame.
	extent _ collapsedFrame notNil
				ifTrue: [collapsedFrame extent]
				ifFalse: [aSSView isMorph
					ifTrue: [aSSView getRawLabel width + aSSView labelWidgetAllowance @ (aSSView labelHeight + 2)]
					ifFalse: [(aSSView labelText extent x + 70) @ aSSView labelHeight
							min: aSSView labelDisplayBox extent]].
	collapsedFrame notNil
		ifTrue: [(otherFrames anySatisfy: [:f | collapsedFrame intersects: f])
				ifFalse: ["non overlapping"
					^ collapsedFrame]].
	top _ viewBox top + verticalBorderDistance.
	[topLeft _ viewBox left @ top.
	newFrame _ topLeft extent: extent.
	newFrame bottom <= (viewBox height - verticalBorderDistance)]
		whileTrue: 
			[(otherFrames anySatisfy: [:w | newFrame intersects: w])
				ifFalse: ["no overlap"
					^ newFrame].
			top _ top + grid].
	"If all else fails... (really to many wins here)"
	^ 0 @ 0 extent: extent! !

!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'di 11/20/2001 00:17'!
assignCollapsePointFor: aSSView
	"Offer up a location along the left edge of the screen for a collapsed SSView.
	Make sure it doesn't overlap any other collapsed frames."

	| grid otherFrames y free topLeft viewBox |
	grid _ 24.  "should be mult of 8, since manual move is gridded by 8"
	aSSView isMorph
		ifTrue: [otherFrames _ (SystemWindow windowsIn: aSSView world satisfying: [:w | true])
					collect: [:w | w collapsedFrame]
					thenSelect: [:rect | rect notNil].
				viewBox _ self reduceByFlaps: aSSView world viewBox]
		ifFalse: [otherFrames _ ScheduledControllers scheduledWindowControllers
					collect: [:aController | aController view collapsedFrame]
					thenSelect: [:rect | rect notNil].
				viewBox _ Display boundingBox].
	y _ viewBox top.
	[(y _ y + grid) <= (viewBox height - grid)]
		whileTrue:
		[topLeft _ viewBox left@y.
		free _ true.
		otherFrames do: [:w | free _ free & (topLeft ~= w topLeft)].
		free ifTrue: [^ topLeft]].
	"If all else fails..."
	^ 0 @ 0! !

!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'di 11/21/2001 14:56'!
reduceByFlaps: aScreenRect
	"Return a rectangle that won't interfere with default shared flaps"

	Flaps sharedFlapsAllowed ifFalse: [^ aScreenRect copy].
	(Flaps globalFlapTabsIfAny allSatisfy: [:ft | ft edgeToAdhereTo == #bottom])
		ifTrue: [^ aScreenRect withHeight: aScreenRect height - 18]
		ifFalse: [^ aScreenRect insetBy: 18]! !

!RealEstateAgent class methodsFor: 'as yet unclassified' stamp: 'di 11/20/2001 00:17'!
strictlyStaggeredInitialFrameFor: aStandardSystemView initialExtent: initialExtent world: aWorld
	"This method implements a staggered window placement policy that I (di) like.
	Basically it provides for up to 4 windows, staggered from each of the 4 corners.
	The windows are staggered so that there will always be a corner visible."

	| allowedArea grid initialFrame otherFrames cornerSel corner delta putativeCorner free maxLevel |

	allowedArea _(self maximumUsableAreaInWorld: aWorld)
		insetBy: (self scrollBarSetback @ self screenTopSetback extent: 0@0).
	(Smalltalk isMorphic and: [Flaps sharedFlapsAllowed]) ifTrue:
		[allowedArea _ self reduceByFlaps: allowedArea].
	"Number to be staggered at each corner (less on small screens)"
	maxLevel _ allowedArea area > 300000 ifTrue: [3] ifFalse: [2].
	"Amount by which to stagger (less on small screens)"
	grid _ allowedArea area > 500000 ifTrue: [40] ifFalse: [20].
	initialFrame _ 0@0 extent: ((initialExtent
							"min: (allowedArea extent - (grid*(maxLevel+1*2) + (grid//2))))
							min: 600@400")).
	otherFrames _ Smalltalk isMorphic
		ifTrue: [(SystemWindow windowsIn: aWorld satisfying: [:w | w isCollapsed not])
					collect: [:w | w bounds]]
		ifFalse: [ScheduledControllers scheduledWindowControllers
				select: [:aController | aController view ~~ nil]
				thenCollect: [:aController | aController view isCollapsed
								ifTrue: [aController view expandedFrame]
								ifFalse: [aController view displayBox]]].
	0 to: maxLevel do:
		[:level | 
		1 to: 4 do:
			[:ci | cornerSel _ #(topLeft topRight bottomRight bottomLeft) at: ci.
			corner _ allowedArea perform: cornerSel.
			"The extra grid//2 in delta helps to keep title tabs distinct"
			delta _ (maxLevel-level*grid+(grid//2)) @ (level*grid).
			1 to: ci-1 do: [:i | delta _ delta rotateBy: #right centerAt: 0@0]. "slow way"
			putativeCorner _ corner + delta.
			free _ true.
			otherFrames do:
				[:w |
				free _ free & ((w perform: cornerSel) ~= putativeCorner)].
			free ifTrue:
				[^ (initialFrame align: (initialFrame perform: cornerSel)
								with: putativeCorner)
						 squishedWithin: allowedArea]]].
	"If all else fails..."
	^ (self scrollBarSetback @ self screenTopSetback extent: initialFrame extent)
		squishedWithin: allowedArea! !


!TextMorph methodsFor: 'initialization' stamp: 'di 11/17/2001 15:34'!
setTextStyle: aTextStyle

	textStyle _ aTextStyle.
	self releaseCachedState; changed! !


!TheWorldMenu methodsFor: 'windows & flaps menu' stamp: 'di 11/21/2001 15:47'!
formulateFlapsMenu: aMenu
	"Fill aMenu with appropriate content"

	aMenu addTitle: 'flaps'.
	aMenu addStayUpItem.
	aMenu addUpdating: #navigatorShowingString enablementSelector: #enableProjectNavigator target: Preferences selector: #togglePreference:  argumentList: #(showProjectNavigator).
	aMenu balloonTextForLastItem: 'Whether to present a pop-up project navigator bar partly off the bottom of the screen.  Suppressed if the Navigator tab is enabled.'.
	Flaps sharedFlapsAllowed
		ifTrue:
			[self fillIn: aMenu from:
				{{#suppressFlapsString.
					{CurrentProjectRefactoring.
					#currentToggleFlapsSuppressed}.
				'Whether prevailing flaps should be shown in the project right now or not.'}}.
			aMenu addWithLabel: '   put them on bottom' enablementSelector: #showSharedFlaps
				target: Flaps selector: #sharedFlapsAlongBottom argumentList: #().
			aMenu balloonTextForLastItem: 'Group all shared flaps along the bottom edge of the screen'.
			aMenu addLine.
			Flaps addIndividualGlobalFlapItemsTo: aMenu].
.
     self fillIn: aMenu from: {
			nil.

               {'make a new flap'.
			{Flaps. #addLocalFlap}.
			'Create a new flap.  You can later make it into a shared flap is you wish.'}.

			nil.}.
	Flaps sharedFlapsAllowed
		ifTrue:
			[self fillIn: aMenu from: {
				{'destroy all shared flaps'.
				{Flaps. #disableGlobalFlaps}.
				'Destroy all the shared flaps and disable their use in all projects.'}}]
		ifFalse:
			[aMenu add: 'install default shared flaps' target: Flaps action: #enableGlobalFlaps.
			aMenu balloonTextForLastItem: 'Create the default set of shared flaps'.
			aMenu addLine].

	self fillIn: aMenu from: {
			nil.
			{'about flaps...'.
			{Flaps . #explainFlaps}.
			'Gives a window full of details about how to use flaps.'}}.
! !

Preferences class removeSelector: #sharedFlapsAlongBottom!
Flaps class removeSelector: #givenID:matches:!
Flaps class removeSelector: #navigatorFlapTab!
Flaps class removeSelector: #replaceFlapwithID:!
Flaps class removeSelector: #replaceNavigatorFlap!
Flaps class removeSelector: #replaceStackToolsFlap!
Flaps class removeSelector: #replaceSuppliesFlap!
Flaps class removeSelector: #replaceWidgetsFlap!
Flaps class removeSelector: #stackToolsFlapTab!
Flaps class removeSelector: #toolsFlapTab!
Flaps class removeSelector: #widgetsFlapTab!
"Postscript:
Convert old instances."
FlapTab allSubInstancesDo: [:ft | ft updateOldInstances].

(Preferences preferenceAt: #showProjectNavigator ifAbsent: [nil])
	ifNotNilDo: [:pref | pref instVarNamed: 'helpString' put:
'showProjectNavigator:
		If true, then show a project navigator control partly off the bottom of the screen.  Suppressed if the Navigator tab is enabled.'].

Preferences okToReinitializeFlaps ifTrue:
	[Flaps addNewDefaultSharedFlaps.
	Flaps sharedFlapsAlongBottom].
Project current assureNavigatorPresenceMatchesPreference.
(Flaps globalFlapTabWithID: 'Tools') ifNotNilDo: [:ft | ft hideFlap].

FlapTab removeSelector: #updateOldInstances.
FlapTab removeSelector: #addNewDefaultSharedFlaps.
!

