'From Squeak 2.3 of January 14, 1999 on 27 January 1999 at 6:33:19 pm'!
"Change Set:		diffInChangeList-sw
Date:			27 January 1999
Author:			Scott Wallace

Deploys Andreas's diff function in changelist & versions browsers.

For a versions browser, each version shows its diffs from the preceding version.

For a changelist browser, diffs from the in-memory versions, if any, are shown.

New preference #showDiffsInChangeList governs whether or not the diff feature should be used."!

StringHolder subclass: #ChangeList
	instanceVariableNames: 'changeList list listIndex listSelections file lostMethodPointer showsVersions '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Changes'!

!ChangeList methodsFor: 'scanning' stamp: 'sw 1/12/1999 12:37'!
scanFile: aFile from: startPosition to: stopPosition
	| itemPosition item prevChar |
	showsVersions _ false.
	file _ aFile.
	changeList _ OrderedCollection new.
	list _ OrderedCollection new.
	listIndex _ 0.
	file position: startPosition.
'Scanning ', aFile localName, '...'
	displayProgressAt: Sensor cursorPoint
	from: startPosition to: stopPosition
	during: [:bar |
	[file position < stopPosition]
		whileTrue:
		[bar value: file position.
		[file atEnd not and: [file peek isSeparator]]
				whileTrue: [prevChar _ file next].
		(file peekFor: $!!)
		ifTrue:
			[prevChar = Character cr ifTrue: [self scanCategory]]
		ifFalse:
			[itemPosition _ file position.
			item _ file nextChunk.
			file skipStyleChunk.
			item size > 0 ifTrue:
				[self addItem: (ChangeRecord new file: file position: itemPosition type: #doIt)
					text: 'do it: ' , (item contractTo: 50)]]]].
	listSelections _ Array new: list size withAll: false! !

!ChangeList methodsFor: 'scanning' stamp: 'sw 1/12/1999 12:37'!
scanVersionsOf: method class: class meta: meta category: category selector: selector
	| position prevPos prevFileIndex preamble tokens sourceFilesCopy stamp |
	showsVersions _ true.
	changeList _ OrderedCollection new.
	list _ OrderedCollection new.
	listIndex _ 0.
	position _ method filePosition.
	sourceFilesCopy _ SourceFiles collect:
		[:x | x isNil ifTrue: [ nil ]
				ifFalse: [x readOnlyCopy]].
	method fileIndex == 0 ifTrue: [^ nil].
	file _ sourceFilesCopy at: method fileIndex.
	[position notNil & file notNil]
		whileTrue:
		[file position: (0 max: position-150).  "Skip back to before the preamble"
		[file position < (position-1)]  "then pick it up from the front"
			whileTrue: [preamble _ file nextChunk].

		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos _ nil.
		stamp _ ''.
		(preamble findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens _ Scanner new scanTokens: preamble]
			ifFalse: [tokens _ Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size-5) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-3) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp _ tokens at: tokens size-2.
						prevPos _ tokens last.
						prevFileIndex _ prevPos // 16r1000000.
						prevPos _ prevPos \\ 16r1000000]
				ifFalse: ["Old format gives no stamp; prior pointer in two parts"
						prevPos _ tokens at: tokens size-2.
						prevFileIndex _ tokens last].
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos _ nil]].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size-3) = #methodsFor:])
			ifTrue:
				[(tokens at: tokens size-1) = #stamp:
				ifTrue: ["New format gives change stamp and unified prior pointer"
						stamp _ tokens at: tokens size]].
 		self addItem:
				(ChangeRecord new file: file position: position type: #method
						class: class name category: category meta: meta stamp: stamp)
			text: stamp , ' ' , class name , (meta ifTrue: [' class '] ifFalse: [' ']) , selector.
		position _ prevPos.
		prevPos notNil ifTrue:
			[file _ sourceFilesCopy at: prevFileIndex]].
	sourceFilesCopy do: [:x | x notNil ifTrue: [x close]].
	listSelections _ Array new: list size withAll: false! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/25/1999 14:46'!
contents
	^ Preferences showDiffsInChangeList
		ifFalse:
			[self undiffedContents]
		ifTrue:
			[self showsVersions
				ifTrue:
					[self diffedVersionContents]
				ifFalse:
					[self contentsDiffedFromCurrent]]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/12/1999 12:40'!
contentsDiffedFromCurrent
	| aChange aClass |
	listIndex = 0
		ifTrue: [^ ''].
	aChange _ changeList at: listIndex.
	^ ((aChange type == #method and: [(aClass _ aChange methodClass) notNil]) and: [aClass includesSelector: aChange methodSelector])
		ifTrue:
			 [Utilities methodDiffFor: aChange text class: aClass selector: aChange methodSelector]
		ifFalse:
			[(changeList at: listIndex) text]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/12/1999 13:54'!
diffedVersionContents
	| thisText |
	listIndex = 0
		ifTrue: [^ ''].
	thisText _ (changeList at: listIndex) text.
	^ listIndex == changeList size
		ifTrue:
			[thisText]
		ifFalse:
			[TextDiffBuilder buildDisplayPatchFrom: (changeList at: (listIndex + 1)) text to: thisText]! !

!ChangeList methodsFor: 'viewing access' stamp: 'sw 1/25/1999 14:45'!
undiffedContents
	^ listIndex = 0
		ifTrue: ['']
		ifFalse: [(changeList at: listIndex) text]! !

!ChangeList methodsFor: 'accessing' stamp: 'sw 1/12/1999 12:38'!
showsVersions
	^ showsVersions == true! !


!Preferences class methodsFor: 'initialization' stamp: 'sw 1/25/1999 13:51'!
chooseInitialSettings 
	"Restore the default choices for Preferences."
	"Preferences chooseInitialSettings"
	#(	(allowSysWindowEmbedding			false)
		(automaticViewerPlacement			true)
		(browseWithPrettyPrint				false)
		(cmdDotEnabled						true)
		(confirmFirstUseOfStyle				true)
		(disableSounds						false)
		(fastDragWindowForMorphic			false)
		(ignoreStyleIfOnlyBold				true)
		(inboardScrollbars					false)
		(logDebuggerStackToFile				true)
		(noviceMode							false)
		(printAlternateSyntax				false)
		(reverseWindowStagger				true)
		(showDebugHaloHandle				true)
		(showDiffsInChangeList				true)
		(showPlayerSource					false)
		(showScriptSource					false)
		(showTimeStampsInMenuTitles		false)
		(showProjectZoom					false)
		(suppressCheckForSlips				false)
		(suppressUpdateServerPrompt		false)
		(thoroughSenders					true)
		(unlimitedPaintArea					false)
		(updateSavesFile						false)
		(useAnnotationPanes				false)
		(useDetailPanesInViewers			false)
		(warnIfNoChangesFile				true)
		(warnIfNoSourcesFile				true))
	do:
		[:aPair |
			aPair last == #true
				ifTrue:
					[self enable: aPair first]
				ifFalse:
					[self disable: aPair first]]

! !

!Preferences class methodsFor: 'initialization' stamp: 'sw 1/25/1999 13:54'!
initializeHelpMessages 
	"Preferences initializeHelpMessages"
  	HelpDictionary _ Dictionary new.
	#(

(allowSysWindowEmbedding
'Determines whether, in Morphic, SystemWindows should automatically be droppable into willing receptors')

(automaticViewerPlacement
'If true, new viewers are automatically positioned near the objects they view; if false, new viewers are attached to the hand, from whence you much choose a destination for them')

(browseWithPrettyPrint
'If true, browsers will automatically format their contents')

(cautionBeforeClosing 
'If true, Morphic windows seen in an mvc project will put up a warning before allowing themselves to be dismissed')

(cmdDotEnabled
'If true, cmd-dot brings up a debugger;
if false, the cmd-dot interrupt is disabled')

(confirmFirstUseOfStyle
'If true, the first attempt to submit a method with non-standard style will bring up a confirmation dialog')

(disableSounds
'If true, all sound playing is disabled')
	
(editPlayerScriptsInPlace 
'If true, textual player scripts are edited in place in Scriptors (still imperfectly implemented)')

(eToyScheme
'If true, new scripting spaces place the Playfield to the left and the the palette to the right of the window; if false, the opposite is true.')

(fastDragWindowForMorphic
'If true, morphic window drag will be done by dragging an outline of the window.')

(ignoreStyleIfOnlyBold
'If true, then any method submission in which the only style change is for bolding will be treated as a method with no style specifications')

(inboardScrollbars
'If true, then ScrollPane will place scrollbars inside on the right and will not hide them on exit')

(logDebuggerStackToFile
'If true, whenever you fall into a debugger a summary of its stack will be written to a file named
''SqueakDebug.log''')

(noviceMode 
'If true, certain novice-mode accommodations are made.')

(printAlternateSyntax
'If true, then
prettyPrint using experimental syntax.
Otherwise use normal ST-80 syntax.')

(reverseWindowStagger
'If true, a reverse-stagger strategy  is used for determining where newly launched windows will be placed; if false, a direct- stagger strategy is used.')

(showDebugHaloHandle 
'If true, a special debugging halo handle is displayed at the right of the halo; if false, no such handle is shown.')

(showDiffsInChangeList
'If true, changeList browsers and Versions browsers reveal the differences between successive versions or between the in-memory code and the code on disk')

(showPlayerSource
'If true, then all Player methods with fewer than 2 arguments are included in Viewers, whether or not they are intended for end-user use.  This can be dangerous')

(showProjectZoom
'If true, then show a zoom effect when entering or leaving projects.  This can be costly of memory (at least an extra screen buffer) so dont use it in low space situations.  But it is cool.')

(showScriptSource
'If true, then the actual Smalltalk source code for methods is shown in the detail panes for scripts in a viewer; if false, then a help message for scripts is shown instead.')

(showTimeStampsInMenuTitles
'If true, then the author''s timestamp is displayed as the menu title of any message list; if false, no author''s timestamps are shown')

(suppressCheckForSlips 
'If false, then whenever you file out a change set, it is checked for ''slips'' and if any are found, you are so informed and given a chance to open a browser on them')

(suppressUpdateServerPrompt
'If true, the prompt for server choice when updating code from the server is suppressed.  Set this to true to leave the server choice unchanged from update to update.')

(thoroughSenders
'If true, then ''senders'' browsers will dive inside structured literals in their search')

(uniformWindowColors
'If true, then all standard windows are given the same color rather than their customized window-type-specific colors')

(unlimitedPaintArea
'If true, the painting area for a new drawing will not be limited in size; if false, a reasonable
limit will be applied, in an attempt to hold down memory and time price.')

(updateRemoveSequenceNum
'If true, then remove the leading sequence number from the filename before automatically saving a local copy of any update loaded.')

(updateSavesFile
'If true, then when an update is loaded from the server, a copy of it will automatically be saved on a local file as well.')

(useAnnotationPanes
'If true, a thin horizontal annotation pane is used in message-list browsers.')

(useDetailPanesInViewers
'If true, then Viewers will have an extra "" control at the left of each row, the hitting of which toggles the appearance of a textual detail pane.')

(useNewViewers
'If true, then the new kinds of viewers introduced in Squeak 2.3 are used; if false, then the old style, from earlier releases, are still used.  Old viewers will hopefully soon be removed from the system.')

(warnIfNoChangesFile
'If true, then you will be warned, whenever you start up, if no changes file
can be found')

(warnIfNoSourcesFile 
'If true, then you will be warned, whenever you start up, if no sources file can be found')) do:
		[:pair | HelpDictionary at: pair first put: 
			(pair first, ':
', pair last)]
! !


!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 1/12/1999 12:24'!
methodDiffFor: aString class: aClass selector: aSelector
	^ (aClass includesSelector: aSelector)
		ifFalse:
			[aString copy]
		ifTrue:
			[TextDiffBuilder buildDisplayPatchFrom: (aClass sourceCodeAt: aSelector) to: aString]! !


"Postscript:
"
Preferences enable: #showDiffsInChangeList.
Preferences initializeHelpMessages.!

