'From Squeak 2.3 of January 14, 1999 on 30 March 1999 at 9:51:16 am'!
"Change Set:		misc-jhm
Date:			30 March 1999
Author:			John Maloney

Misc things for the 2.4 release including:
  1. new Mac project files for building the VM
  2. cache colormaps used for translating between 16 and 32 bit Forms
  3. faster mapColor:to: (using the colormap caching above)
  3. change sound setLoudness to loudness:
"!


!AIFFFileReader methodsFor: 'reading' stamp: 'jm 3/24/1999 10:39'!
readFromFile: fileName mergeIfStereo: mergeFlag skipDataChunk: skipDataFlag
	"Read the AIFF file of the given name. If mergeFlag is true and the file contains stereo data, then the left and right channels will be mixed together as the samples are read in. If skipDataFlag is true, then the data chunk to be skipped; this allows the other chunks of a file to be processed in order to extract format information quickly without reading the data."
	"AIFFFileReader new readFromFile: 'test.aiff' mergeIfStereo: false skipDataChunk: true"

	| f |
	mergeIfStereo _ mergeFlag.
	skipDataChunk _ skipDataFlag.
	f _ (FileStream readOnlyFileNamed: fileName) binary.
	isLooped _ false.
	gain _ 1.0.
	self readFrom: f.
	f close.
! !


!AbstractSound methodsFor: 'initialization' stamp: 'jm 3/24/1999 12:03'!
loudness: aNumber
	"Initialize my volume envelopes and initial volume. Subclasses overriding this method should include a resend to super."

	| vol |
	vol _ (aNumber asFloat max: 0.0) min: 1.0.
	envelopes do: [:e |
		(e isKindOf: VolumeEnvelope) ifTrue: [e scale: vol]].
	self initialVolume: vol.
! !


!Color class methodsFor: 'colormaps' stamp: 'jm 3/25/1999 20:06'!
cachedColormapFrom: sourceDepth to: destDepth
	"Return a cached colormap for mapping between the given depths. Always return a real colormap, not nil; this allows the client to get an identity colormap that can then be copied and modified to do color transformations."
	"Note: This method returns a shared, cached colormap to save time and space. Clients that need to modify a colormap returned by this method should make a copy and modify that!!"
	"Note: The colormap cache may be cleared by evaluating 'Color shutDown'."

	| key newMap |
	key _ sourceDepth@destDepth.
	CachedColormaps == nil ifTrue: [CachedColormaps _ Dictionary new].
	^ CachedColormaps at: key ifAbsent: [
		newMap _ self computeColormapFrom: sourceDepth to: destDepth.
		CachedColormaps at: key put: newMap.
		sourceDepth >= 16 ifTrue: [
			"can use the same map from both 16-bits and 32-bits to a given depth"
			CachedColormaps at: 16@destDepth put: newMap.
			CachedColormaps at: 32@destDepth put: newMap].
		newMap].
! !

!Color class methodsFor: 'colormaps' stamp: 'jm 3/25/1999 19:48'!
computeColormapFrom: sourceDepth to: destDepth
	"Compute a colorMap for translating between the given depths. A colormap is a Bitmap whose entries contain the pixel values for the destination depth. Typical clients use cachedColormapFrom:to: instead."

	| map bitsPerColor |
	sourceDepth < 16 ifTrue: [
		"source is 1-, 2-, 4-, or 8-bit indexed color"
		map _ (IndexedColors copyFrom: 1 to: (1 bitShift: sourceDepth))
					collect: [:c | c pixelValueForDepth: destDepth].
		map _ map as: Bitmap.
	] ifFalse: [
		"source is 16-bit or 32-bit RGB"
		destDepth > 8
			ifTrue: [bitsPerColor _ 5]  "retain maximum color resolution"
			ifFalse: [bitsPerColor _ 4].
		map _ self computeRGBColormapFor: destDepth bitsPerColor: bitsPerColor].

	"Note: zero is transparent except when source depth is one-bit deep"
	sourceDepth > 1 ifTrue: [map at: 1 put: 0].
	^ map
! !


!FlashMorph methodsFor: 'accessing' stamp: 'jm 3/26/1999 06:19'!
imageFormAALevel: aNumber color: aColor

	| canvas |
	self defaultAALevel: aNumber.
	canvas _ FormCanvas extent: self extent depth: 32.
	canvas fillRectangle: (0@0 extent: self extent) color: aColor.
	self fullDrawOn: (canvas copyOffset: self topLeft negated).
	^ canvas form mapColor: aColor to: Color transparent! !


!Form methodsFor: 'other' stamp: 'jm 3/25/1999 20:19'!
mapColor: oldColor to: newColor
	"Make all pixels of the given color in this Form to the given new color."
	"Warnings: This method modifies the receiver. It may lose some color accuracy on 32-bit Forms, since the transformation uses a color map with only 15-bit resolution."

	| map |
	map _ (Color cachedColormapFrom: depth to: depth) copy.
	map at: (oldColor indexInMap: map) put: (newColor pixelWordForDepth: depth).
	(BitBlt toForm: self)
		sourceForm: self;
		sourceOrigin: 0@0;
		combinationRule: Form over;
		destX: 0 destY: 0 width: width height: height;
		colorMap: map;
		copyBits.
! !


!InterpreterSupportCode class methodsFor: 'source files' stamp: 'jm 3/29/1999 18:18'!
macArchiveBinaryFile
	"To create this method use:
	  (FileStream oldFileNamed: 'projectArchive.sit') binary contentsOfEntireFile asArray"

	^ #(83 73 84 33 0 3 0 0 51 52 114 76 97 117 2 146 0 0 0 22 7 66 13 13 14 83 113 117 101 97 107 54 56 75 46 112 114 111 106 46 112 114 111 106 0 0 0 0 0 0 0 0 0 0 0 0 186 147 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 19 168 0 0 0 0 255 255 255 255 77 77 80 82 67 87 73 69 1 0 172 94 90 110 179 37 114 175 0 0 41 239 0 0 39 248 0 0 9 121 0 0 9 169 25 238 155 248 0 0 0 0 0 0 234 36 13 192 248 163 215 42 83 182 114 106 29 97 132 157 132 230 115 60 119 144 110 237 86 46 225 70 216 111 74 158 187 214 202 188 236 117 179 149 208 223 244 146 19 238 37 190 151 221 42 211 243 226 248 167 125 23 217 132 173 149 117 70 110 27 161 155 76 182 158 183 37 108 159 146 77 216 109 178 201 62 182 37 251 9 35 156 108 194 54 219 70 9 37 236 54 97 132 19 74 88 214 177 253 198 178 75 114 219 216 246 34 207 201 30 120 238 103 43 240 213 213 109 122 245 146 45 221 210 162 39 95 217 182 99 30 97 155 28 63 9 143 60 39 183 143 236 103 120 178 1 58 129 0 244 242 0 12 128 3 128 235 79 2 204 193 127 222 100 209 220 162 137 49 89 138 26 98 66 150 84 209 140 73 166 104 104 9 89 83 101 81 142 27 133 13 119 181 201 82 235 194 197 171 124 73 93 219 180 138 199 234 193 96 168 62 208 188 162 154 131 149 179 207 161 191 193 211 175 173 175 172 187 93 5 40 123 9 25 247 200 98 56 46 69 90 69 173 205 140 43 72 144 40 27 22 103 51 166 24 34 254 204 152 44 70 229 13 82 91 220 196 189 209 106 106 73 49 169 43 170 41 235 190 21 141 236 240 224 33 49 34 169 170 102 138 97 100 163 109 150 163 162 182 97 131 77 194 170 19 150 35 82 155 33 139 138 73 244 80 16 30 92 184 155 205 26 225 175 203 5 60 22 202 67 164 3 119 151 106 142 44 30 89 184 29 0 88 100 214 74 82 164 40 40 69 196 213 13 226 194 197 173 98 224 230 192 188 121 190 111 158 171 226 167 77 190 189 177 247 31 229 58 184 2 254 99 131 117 124 48 209 70 198 117 246 188 90 201 7 19 56 72 167 51 112 58 205 209 34 168 220 218 210 218 185 19 96 73 154 62 91 193 2 87 80 83 165 136 54 194 52 31 42 192 205 61 198 167 250 246 175 46 70 143 236 9 96 42 112 220 77 252 45 252 103 243 58 192 11 80 117 8 224 115 223 193 115 37 212 199 219 152 58 231 119 249 197 172 53 120 236 119 55 214 175 132 22 196 215 158 125 8 90 110 120 222 194 0 144 72 139 63 245 147 179 248 255 149 63 229 79 253 244 196 156 181 168 182 23 142 99 229 237 196 181 137 148 168 243 183 127 254 40 180 220 84 11 107 127 187 30 82 39 123 33 117 223 86 127 106 151 155 157 73 237 216 100 49 177 248 137 69 254 118 170 231 111 239 58 9 119 252 232 254 115 92 16 73 125 114 175 45 113 70 243 20 60 103 247 4 241 27 104 244 246 253 229 254 246 175 225 105 155 31 76 89 7 199 241 63 200 231 190 231 160 101 201 41 220 127 139 113 57 204 56 191 4 59 153 210 83 176 42 50 230 84 224 248 202 74 81 232 122 230 83 16 169 59 4 165 235 60 66 217 99 30 196 59 96 250 253 131 172 105 9 63 47 4 240 46 58 10 94 113 42 120 143 189 12 69 124 250 149 245 124 250 204 20 136 47 56 41 116 9 93 127 44 22 186 126 118 0 188 223 95 42 148 125 253 148 240 176 154 162 51 224 125 162 20 148 223 205 228 62 244 107 60 255 76 175 208 245 172 8 188 119 247 130 147 176 9 188 79 30 20 186 126 113 152 180 3 216 237 1 248 226 250 179 86 135 3 214 241 243 27 139 103 226 62 149 62 178 178 14 29 101 208 17 132 174 211 31 17 202 119 49 63 19 202 244 19 248 103 250 49 124 90 241 12 106 83 58 143 245 15 96 173 90 132 59 176 198 97 252 247 226 191 27 255 71 185 103 129 47 171 12 202 166 174 109 145 245 86 67 108 48 37 53 42 233 81 177 78 9 235 146 126 119 101 176 161 78 12 84 34 163 206 125 110 232 20 94 124 122 7 242 114 91 251 10 132 42 240 228 222 63 164 187 75 31 177 48 60 210 57 151 170 149 166 187 111 187 103 168 222 190 27 207 213 219 157 194 51 181 233 238 91 77 166 93 231 183 15 11 165 247 46 99 255 47 221 216 1 220 130 222 116 247 244 79 128 87 169 75 63 177 21 253 78 169 19 202 142 158 198 154 251 211 61 233 52 213 193 170 165 112 125 211 65 152 254 104 71 186 103 251 250 116 207 14 226 116 8 29 162 229 204 110 68 62 67 188 88 219 184 133 50 252 91 213 110 103 174 58 5 187 177 18 3 6 230 139 134 182 100 82 211 205 202 123 65 40 95 254 32 99 79 21 59 247 221 0 237 118 15 236 250 123 15 236 250 225 252 119 123 32 180 255 124 15 108 219 203 254 195 61 240 93 82 198 238 129 109 232 160 255 234 1 12 20 79 129 139 82 22 11 89 171 176 253 213 86 89 127 61 161 216 21 66 161 192 95 42 64 241 109 66 231 158 19 248 47 218 14 124 77 250 7 47 179 252 148 15 123 230 3 255 203 10 40 169 10 133 208 201 254 187 200 154 171 146 201 184 51 0 143 96 176 110 105 174 78 33 154 161 184 34 65 206 64 210 191 81 35 24 170 171 113 8 224 209 28 64 55 115 2 208 181 190 161 126 136 224 123 60 26 171 215 52 230 249 194 177 8 118 52 235 96 230 78 123 138 133 206 135 123 201 105 160 162 36 238 89 42 197 227 154 166 138 181 114 60 105 219 9 214 0 58 47 20 156 102 169 133 120 240 190 200 40 95 79 176 217 74 247 195 221 97 241 200 247 69 230 205 115 4 224 145 231 139 140 10 242 15 30 182 236 14 0 186 38 71 136 92 230 33 111 29 21 228 157 101 245 107 236 63 253 90 212 199 35 171 198 53 200 35 36 25 17 41 126 65 88 75 215 164 100 56 5 240 136 140 40 155 211 149 12 141 120 184 136 199 91 78 52 206 131 247 157 207 81 3 43 175 94 190 231 82 191 26 163 62 237 66 25 170 140 4 243 105 187 236 235 62 189 38 176 250 98 142 41 162 34 53 217 36 19 213 34 109 255 179 32 150 63 116 67 207 134 134 49 30 70 44 30 206 213 41 12 85 191 173 42 242 48 219 28 2 120 208 202 247 97 151 252 91 215 16 245 98 46 179 59 160 36 114 148 254 141 26 87 18 249 48 15 199 128 68 190 122 233 74 186 155 207 234 17 12 53 231 244 112 250 72 246 111 170 122 37 217 103 121 92 78 246 23 121 140 37 251 139 186 142 37 251 28 15 231 133 100 63 164 199 63 147 253 176 176 239 37 251 28 143 177 100 63 68 227 253 100 159 229 225 28 144 236 243 125 134 188 241 45 159 134 106 8 34 126 202 194 175 197 153 165 1 159 126 133 13 196 56 127 36 242 55 27 255 74 34 191 224 176 142 1 137 188 89 81 23 220 34 110 93 188 240 131 68 238 126 55 81 33 209 81 231 24 79 84 89 30 151 147 208 69 61 198 146 80 142 199 184 211 15 241 120 223 233 179 60 242 124 89 87 156 136 60 208 24 46 70 247 171 14 212 207 35 174 140 216 226 192 205 199 133 78 35 30 90 24 135 185 46 240 88 29 222 148 227 81 16 28 30 23 185 228 136 19 131 193 0 206 47 57 1 172 191 57 235 68 84 139 117 252 219 60 208 201 226 35 153 117 50 241 136 75 134 33 6 180 68 82 137 191 166 126 198 57 176 227 229 13 142 0 186 94 117 142 254 76 159 189 27 24 231 145 115 232 92 159 245 59 171 57 210 102 227 203 133 19 143 12 224 55 191 51 251 57 97 147 22 83 19 82 66 254 128 45 50 228 104 195 227 148 165 195 203 162 70 112 126 233 129 190 53 13 38 224 168 53 243 33 54 51 140 160 61 213 75 243 184 182 107 224 42 4 97 176 131 0 122 240 4 119 141 53 169 46 54 5 71 47 49 134 87 77 85 67 163 117 219 239 59 85 132 227 179 87 215 18 92 125 224 159 167 173 193 187 187 79 65 134 224 90 12 5 25 255 197 209 133 90 156 203 151 117 99 44 114 120 216 148 122 127 103 252 24 35 226 224 9 139 171 221 39 77 193 144 174 36 12 252 43 166 178 89 54 232 239 24 99 68 217 117 231 157 134 41 233 108 16 213 185 97 209 12 244 84 234 109 219 22 7 22 121 37 139 21 236 96 55 184 156 0 172 108 137 3 132 226 220 52 246 102 131 51 205 122 148 166 252 115 47 64 80 22 177 178 13 135 23 7 244 167 55 33 214 56 2 164 156 92 227 254 41 5 13 189 56 65 94 134 111 46 140 19 226 61 88 43 196 174 15 44 135 115 120 89 83 231 6 86 47 171 158 43 206 93 86 213 88 133 219 208 138 64 227 92 167 149 22 240 208 139 35 253 235 220 235 54 174 146 23 253 51 78 227 145 82 238 203 192 39 155 216 45 99 73 82 199 43 69 156 239 111 104 172 159 133 21 183 15 208 209 95 96 87 116 44 109 212 31 129 107 113 235 35 108 249 3 64 87 113 139 9 171 13 1 93 229 30 37 44 121 12 112 30 143 43 33 44 177 13 234 17 99 243 130 139 30 7 188 96 230 238 32 204 251 16 40 136 225 11 3 0 179 142 3 190 134 131 47 75 224 218 252 8 124 28 177 51 132 149 172 128 7 208 221 138 9 187 25 95 72 64 9 110 37 108 246 118 248 2 192 140 169 132 149 22 195 87 17 43 39 108 218 34 56 128 175 234 60 72 88 225 247 48 231 65 25 190 151 0 80 189 3 158 68 140 57 187 26 131 83 136 29 35 44 174 195 11 248 8 75 23 97 194 211 240 123 196 78 16 182 176 16 95 59 129 242 231 9 187 43 204 77 6 16 123 8 211 230 115 101 136 97 23 225 233 131 28 246 128 136 245 113 100 102 21 247 97 128 235 190 193 180 63 194 85 1 204 124 156 176 235 78 112 65 84 247 211 132 85 236 229 240 221 132 89 7 89 67 29 166 215 96 230 176 26 51 82 132 113 79 177 158 8 225 107 66 114 196 20 177 39 13 69 83 11 217 245 127 223 101 240 36 26 16 9 104 81 121 185 172 186 251 135 69 216 49 187 178 215 30 121 202 148 204 224 205 146 174 42 234 70 195 85 29 85 76 77 207 175 209 84 179 48 195 182 122 171 169 75 134 55 208 102 152 90 66 92 37 223 189 69 211 163 134 167 42 18 145 241 42 58 36 153 49 195 213 40 233 27 101 211 179 180 77 137 71 237 130 5 77 1 124 254 66 54 219 146 147 104 150 212 22 166 136 176 101 138 129 183 216 114 34 140 18 184 251 7 77 89 29 155 117 161 53 34 145 17 202 134 251 116 10 212 4 177 216 36 122 54 34 67 140 176 65 98 253 111 74 8 4 225 192 14 171 103 19 244 88 163 203 168 159 42 199 61 56 53 213 199 245 255 0 13 0 107 156 230 36 63 243 27 216 231 32 183 167 53 160 87 231 203 41 31 216 81 217 218 217 201 78 60 61 59 143 217 117 118 26 158 122 210 199 111 182 190 133 255 250 246 145 219 76 242 91 50 216 108 147 217 38 183 201 111 50 218 246 90 170 236 247 145 231 23 153 108 178 143 252 38 191 201 111 114 91 178 79 178 201 126 178 201 38 156 252 62 37 219 35 131 77 126 179 77 38 155 2 247 182 178 223 20 32 250 92 207 1 243 194 111 182 159 176 75 50 225 4 13 155 146 141 39 43 240 5 148 131 110 141 24 70 140 16 226 129 223 109 132 236 249 58 252 94 128 94 16 126 139 225 119 148 20 145 125 240 219 88 26 50 82 102 68 179 136 231 131 95 145 47 147 142 209 153 107 109 164 20 16 232 87 134 53 253 16 175 74 81 35 59 187 245 33 83 53 117 64 237 100 120 37 136 33 112 57 30 253 109 172 246 246 132 2 215 1 207 185 120 132 28 184 76 42 78 213 16 207 239 190 65 200 185 25 207 201 245 31 145 131 36 8 53 229 239 58 28 157 129 225 225 71 249 99 117 243 249 162 106 82 236 137 212 159 184 137 199 48 15 21 189 124 110 134 252 133 60 68 8 76 179 145 116 49 133 80 141 48 100 74 1 144 174 39 108 205 76 54 69 28 120 231 19 0 127 32 127 132 170 222 153 163 218 26 243 182 7 125 74 208 212 227 214 237 105 84 172 253 233 171 211 95 133 41 170 210 111 158 247 66 125 4 251 107 176 174 75 255 222 46 130 186 26 234 82 168 187 240 93 39 212 111 64 93 130 117 41 214 101 88 151 99 93 153 254 226 111 127 2 117 0 251 71 177 46 254 225 163 3 111 65 253 34 212 23 160 62 132 117 55 214 14 191 76 170 132 84 89 99 126 61 166 253 139 94 178 123 213 11 126 255 193 136 221 236 240 187 193 26 3 175 243 90 103 19 145 21 158 59 238 240 187 97 122 199 95 105 191 254 226 174 41 202 143 203 119 45 242 232 242 254 44 206 243 63 254 255 70 254 14 236 109 114 248 175 97 252 183 235 166 22 177 13 243 108 94 226 224 166 203 252 175 94 193 187 163 123 244 237 122 198 123 199 36 132 128 132 26 43 216 91 159 228 61 136 188 172 158 193 255 159 200 71 212 230 28 221 175 99 252 31 49 206 90 182 30 57 83 112 118 129 235 232 126 45 242 179 106 132 191 123 74 60 52 199 54 44 228 51 224 185 71 179 39 12 243 54 205 228 49 201 243 30 244 115 193 239 186 233 203 215 104 191 98 250 205 75 215 160 14 165 63 62 191 109 129 119 193 119 35 234 227 62 239 91 16 207 225 125 51 240 123 17 234 141 88 187 58 248 192 243 27 96 113 191 163 247 205 76 7 33 205 212 213 152 55 17 85 2 237 129 160 97 218 243 138 104 195 250 20 242 33 244 176 9 251 194 255 14 99 253 248 244 47 63 190 233 135 79 79 255 160 139 246 125 248 94 232 104 63 190 115 242 146 208 149 240 207 87 81 255 10 190 239 89 242 213 103 112 30 225 51 175 35 110 47 226 58 126 123 18 231 168 65 220 245 216 127 5 223 159 198 218 155 190 250 211 175 221 208 237 94 228 247 53 196 115 253 250 192 29 223 30 68 126 132 190 247 33 31 174 174 191 226 153 2 85 54 58 186 174 228 186 54 82 137 232 202 46 227 11 210 223 158 68 158 132 174 179 58 118 116 251 20 234 53 171 95 87 167 205 136 227 232 179 9 223 9 223 116 116 234 232 241 9 172 179 122 116 117 248 185 254 78 121 78 51 21 201 124 1 251 17 208 223 113 61 17 53 38 150 5 248 232 146 254 178 254 40 244 212 138 245 214 233 75 223 165 117 59 226 184 57 68 232 206 209 217 115 56 111 75 250 195 221 223 190 161 179 231 239 229 21 39 151 30 196 254 195 75 249 229 158 190 10 251 39 69 183 34 119 247 79 47 173 214 177 95 91 111 141 245 198 162 204 191 87 56 126 97 181 14 126 33 127 255 155 92 248 94 94 245 130 223 239 17 186 153 174 114 252 191 214 26 27 208 77 59 165 198 32 6 140 234 9 45 203 113 191 228 215 225 245 115 30 29 254 238 236 57 215 68 19 230 221 101 119 6 14 54 133 114 207 149 205 123 187 176 222 38 115 202 35 216 223 137 181 163 91 145 71 156 156 215 128 245 14 172 183 203 124 33 246 36 66 199 206 126 196 79 186 30 250 51 212 130 71 212 105 131 162 218 163 112 160 224 243 43 45 173 71 119 54 30 208 155 91 163 123 155 224 133 228 63 44 116 186 56 71 157 114 156 143 208 159 74 216 122 92 91 152 128 127 199 86 230 168 13 154 198 48 44 156 205 150 214 51 48 80 110 12 50 176 50 71 101 128 222 49 12 171 17 45 55 1 255 142 175 204 177 85 9 117 239 244 53 161 60 252 170 153 85 136 184 183 200 242 209 151 130 125 98 24 148 113 135 143 155 143 217 20 223 252 177 187 147 4 133 127 223 51 21 238 56 17 166 29 192 100 168 174 179 81 95 200 160 94 132 61 54 67 117 247 222 114 214 98 142 90 108 141 53 141 22 72 165 25 34 131 250 22 89 211 6 168 71 90 74 226 224 61 5 92 230 252 12 149 117 74 56 106 169 101 71 117 35 15 203 124 95 160 190 65 74 57 106 25 160 198 244 161 28 172 68 45 203 160 154 122 98 228 62 106 57 242 202 12 112 153 215 155 168 21 109 112 67 208 85 87 9 186 235 210 212 168 102 90 183 176 217 94 131 161 178 206 90 138 218 217 90 222 174 89 103 14 167 22 33 225 99 169 128 161 178 78 37 71 45 13 159 77 106 57 76 96 136 102 8 206 16 237 84 113 212 58 159 145 136 234 182 110 192 101 0 144 101 26 114 2 150 136 56 42 237 84 115 212 181 138 62 169 69 21 35 186 36 9 150 123 56 42 237 172 227 168 101 29 227 90 194 206 83 196 210 22 87 3 237 212 224 172 204 101 163 166 58 177 8 204 50 21 71 165 157 245 28 117 93 6 53 172 77 218 243 200 82 67 181 168 188 222 208 49 91 143 229 40 146 26 170 67 94 21 45 78 125 53 231 40 52 13 10 53 20 145 122 212 43 189 193 41 232 85 50 180 65 104 200 52 141 37 55 97 106 160 201 149 171 129 118 54 114 47 47 3 105 135 125 193 28 170 204 13 155 80 121 222 100 50 166 133 213 216 153 69 96 201 208 102 76 27 222 104 212 212 44 235 68 183 74 109 101 14 90 206 218 128 179 246 107 22 191 219 251 239 172 91 132 231 153 106 242 129 134 182 162 202 59 53 203 86 99 11 174 128 26 162 169 158 107 136 118 182 33 170 223 136 229 140 124 126 214 135 145 215 48 156 45 22 60 69 162 110 23 168 96 90 15 80 119 160 55 132 34 166 158 204 209 234 160 238 68 167 132 84 29 43 168 79 162 238 18 106 208 39 233 154 225 158 26 30 65 189 6 34 70 226 129 94 119 115 71 113 239 242 192 71 222 206 56 202 219 176 183 102 168 238 181 0 96 189 147 65 125 7 118 214 98 214 229 29 183 68 125 140 163 186 151 55 18 117 79 38 231 141 235 57 199 155 243 242 70 244 6 216 166 0 168 117 207 41 247 162 242 184 16 114 202 99 23 6 124 86 218 121 156 163 86 250 140 120 210 72 228 82 155 68 125 2 81 217 172 129 68 50 181 232 69 18 245 73 142 90 197 210 70 62 34 74 94 159 226 168 37 29 193 160 175 144 203 101 166 108 66 134 0 148 26 243 144 49 121 11 91 206 218 204 81 171 123 12 91 31 214 35 139 185 102 14 117 31 170 156 155 73 94 229 179 249 245 105 142 90 1 185 220 54 141 92 20 149 168 207 160 202 21 45 145 223 27 204 161 238 23 62 4 135 82 198 200 125 134 14 32 67 52 163 116 192 238 96 9 86 170 225 160 72 108 176 71 49 63 75 0 45 20 213 255 243 138 36 110 16 151 128 217 125 7 79 216 180 243 44 242 26 98 155 158 251 137 173 85 228 114 80 222 131 140 242 28 162 6 141 137 219 188 74 47 127 94 184 114 4 2 236 103 179 190 192 211 134 123 71 1 25 227 74 38 109 92 33 47 114 212 10 107 12 60 101 88 207 110 190 0 235 66 6 245 2 57 196 81 225 118 52 24 83 237 97 195 140 135 146 90 132 218 223 63 241 37 234 75 215 83 212 237 75 22 192 186 156 65 189 12 247 16 34 69 45 223 79 0 214 165 12 234 37 184 222 99 168 238 181 159 204 94 94 145 149 25 96 97 11 8 151 113 184 207 131 206 97 204 202 144 135 226 15 178 178 143 163 214 250 250 21 62 238 98 224 147 168 237 200 80 183 49 1 139 165 252 166 103 214 149 59 184 26 220 27 111 185 235 245 243 237 102 165 154 176 244 193 164 106 2 216 167 243 202 173 109 167 216 164 234 95 210 6 243 251 6 185 19 237 226 168 37 137 84 44 175 187 185 89 3 28 181 124 92 29 140 233 86 110 88 57 235 17 142 90 26 213 199 11 4 57 179 30 197 89 39 34 163 170 153 199 149 179 118 115 229 205 95 199 204 194 75 167 84 80 229 71 225 40 91 203 235 65 230 134 30 142 90 135 35 118 76 70 180 36 77 163 82 182 210 81 122 69 126 85 227 90 191 54 2 50 186 181 173 148 179 6 51 158 23 213 252 166 58 18 95 76 230 114 214 190 76 110 88 190 57 151 91 192 254 172 134 102 83 128 220 244 132 248 150 189 230 250 150 157 223 116 56 122 165 87 131 83 138 226 235 98 157 48 69 245 253 218 189 146 73 95 125 247 125 207 148 18 236 134 43 166 119 223 39 199 40 106 123 155 123 19 147 190 250 158 38 80 223 211 200 0 155 181 193 189 128 129 212 10 137 141 161 66 135 28 103 168 117 238 189 75 186 239 23 109 158 41 203 78 13 209 14 57 65 221 203 215 236 94 183 164 63 249 214 143 112 86 232 144 87 184 202 55 100 110 89 22 87 62 114 243 253 42 38 155 64 92 29 97 136 116 227 59 191 21 145 168 39 57 106 85 95 123 239 240 176 21 49 53 237 211 53 138 227 40 175 161 123 133 108 53 17 85 205 40 205 53 119 183 47 175 139 221 188 49 14 255 210 146 207 40 179 12 157 226 168 229 222 152 174 230 54 108 115 179 14 98 154 31 208 163 90 225 138 69 250 235 105 142 90 147 81 158 146 178 100 186 94 96 72 205 100 229 229 255 98 145 78 57 196 81 221 139 165 43 129 38 116 20 218 129 255 131 97 168 255 249 255 24 62 235 141 62 252 71 15 41 10 134 180 145 153 153 204 35 120 3 71 245 240 6 74 82 178 229 147 239 64 127 59 253 231 154 96 183 101 83 0 120 3 151 135 181 63 131 227 202 15 131 254 152 69 223 252 31 0 0 0 13 13 14 83 113 117 101 97 107 80 80 67 46 112 114 111 106 46 104 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 136 181 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 22 0 0 39 230 0 0 0 0 255 255 255 255 77 77 80 82 67 87 73 69 1 0 172 94 103 33 179 37 114 180 0 0 42 205 0 0 39 248 0 0 10 81 0 0 9 125 106 249 178 140 0 0 0 0 0 0 54 112 13 192 248 163 239 49 175 149 233 177 219 148 123 110 154 61 158 13 210 222 227 43 187 205 46 194 189 86 194 205 181 179 223 52 57 181 147 211 207 227 148 167 23 197 83 195 115 122 214 154 207 87 139 31 237 178 110 45 194 202 51 242 155 73 56 225 13 88 81 254 148 236 179 35 155 29 97 100 235 62 178 223 139 145 219 70 201 62 74 40 161 21 35 91 74 54 187 205 54 219 38 109 179 58 201 182 211 65 157 132 17 126 110 70 248 137 7 190 234 146 211 44 47 207 45 45 154 121 132 109 194 60 194 54 185 173 155 109 242 155 252 38 251 25 158 108 96 188 103 122 121 120 224 3 224 0 32 229 14 128 153 120 70 221 237 81 14 47 94 195 129 242 166 15 174 179 42 71 240 124 187 108 195 92 179 114 64 189 229 199 23 221 93 39 137 43 29 14 155 217 231 247 214 0 143 213 237 118 71 161 173 116 105 22 7 185 113 61 217 195 13 248 252 68 108 89 81 53 21 244 174 23 1 230 212 78 95 98 86 126 183 233 177 220 172 187 116 203 54 108 122 218 50 5 51 73 57 30 165 121 174 25 76 120 26 240 28 149 191 237 29 93 193 235 96 42 55 44 219 64 111 125 186 205 133 197 5 243 98 242 14 37 31 81 124 249 43 242 235 230 1 196 48 108 49 236 139 142 128 37 114 123 34 58 6 3 242 75 0 114 94 229 18 103 226 11 30 116 244 142 81 73 247 168 132 103 240 174 82 75 193 235 125 162 115 165 88 37 157 226 82 141 209 161 247 102 121 85 237 233 247 99 138 86 201 1 169 214 230 22 101 89 146 207 64 152 241 62 83 167 67 7 60 22 138 194 76 35 222 46 214 236 15 30 81 96 207 106 6 68 46 49 98 178 139 78 161 160 72 64 137 16 108 115 109 169 169 230 239 246 84 177 170 109 166 94 66 247 101 92 35 23 203 223 119 188 14 182 126 95 23 247 134 225 68 37 51 196 160 148 41 225 180 162 112 20 148 154 172 94 41 54 110 4 88 162 208 23 98 48 86 103 247 122 68 167 183 31 105 52 204 130 56 238 71 124 176 235 126 34 24 60 85 65 18 128 227 110 228 94 231 55 71 77 3 35 64 122 11 192 23 127 136 239 146 168 143 63 195 216 57 117 139 78 100 173 193 99 191 199 97 125 11 148 97 126 249 177 39 160 108 206 239 213 28 0 2 41 179 6 127 117 12 207 67 214 160 53 216 185 231 218 229 200 182 17 118 97 229 181 132 21 85 14 184 60 107 195 227 59 160 236 186 28 88 254 102 57 4 247 118 64 240 129 122 107 240 177 56 246 38 184 174 70 69 162 226 19 76 214 6 170 103 109 104 218 11 119 252 242 161 30 44 152 9 62 248 141 16 197 97 206 131 240 74 168 39 8 95 119 163 55 108 157 108 109 120 26 95 135 240 193 184 21 176 11 207 227 120 30 120 5 202 150 236 195 251 247 24 150 118 134 249 13 216 200 152 30 135 85 17 49 231 1 142 183 88 4 125 211 203 235 193 153 215 2 201 43 12 250 201 49 251 48 223 8 19 30 58 142 154 66 255 106 60 128 113 225 14 48 10 9 96 252 217 17 48 241 202 159 203 121 229 232 56 112 47 216 171 111 210 55 189 147 168 111 250 237 179 96 124 62 67 63 233 91 251 244 95 245 4 233 13 24 119 38 131 235 173 105 220 236 195 248 254 229 14 125 211 111 4 224 141 205 11 246 162 31 48 254 252 57 125 211 31 218 137 59 128 102 3 192 87 202 143 133 60 2 214 177 242 85 137 211 240 30 84 182 231 230 161 162 28 87 4 125 211 107 249 250 201 79 154 88 110 146 127 15 158 140 63 150 31 159 56 145 218 148 222 99 253 103 177 86 14 166 27 177 70 59 158 29 120 182 226 185 3 21 133 55 90 80 92 81 91 139 234 124 62 175 63 96 33 109 179 2 87 115 160 3 18 226 219 97 236 83 179 34 2 148 166 118 37 86 229 106 14 206 131 132 101 251 240 220 85 173 188 101 197 252 108 146 67 72 184 38 13 115 71 240 52 193 184 134 98 172 223 50 3 162 163 49 47 188 91 31 241 29 116 168 249 180 0 151 250 147 102 217 233 119 193 132 76 84 182 107 210 152 170 214 28 66 182 145 31 86 127 20 214 39 62 18 35 29 48 253 141 181 214 71 191 19 192 12 10 1 225 57 244 77 72 152 178 133 203 52 56 48 183 27 241 53 50 201 28 251 212 26 181 62 242 65 245 19 166 250 66 8 18 97 42 169 37 87 8 252 36 139 93 10 248 189 247 74 254 149 178 80 20 16 61 149 162 191 82 200 115 85 248 69 255 42 139 189 40 79 176 89 176 210 11 123 226 224 97 253 193 95 175 195 234 113 234 157 154 99 22 190 252 197 219 74 107 242 183 213 28 62 121 184 198 135 213 146 149 214 155 87 247 214 219 147 214 83 79 103 194 55 57 74 235 245 196 40 190 217 223 206 57 142 182 208 169 111 118 150 3 183 160 67 105 157 240 89 48 186 242 148 157 245 141 116 231 110 77 172 193 154 91 149 54 236 22 172 131 85 29 48 179 228 57 152 176 173 81 105 91 91 174 180 173 11 226 251 22 84 201 178 163 205 152 249 28 225 98 204 199 97 37 58 169 218 237 106 51 36 188 4 58 114 89 33 147 149 231 242 172 148 252 39 29 202 80 5 24 223 198 189 176 115 15 158 166 181 192 103 43 187 143 48 255 20 189 99 231 60 224 247 207 130 164 116 135 3 149 236 191 65 210 156 238 243 185 181 1 112 216 237 121 25 131 117 226 81 8 132 165 181 164 142 68 253 153 26 118 71 94 182 70 0 142 82 27 138 160 22 0 175 133 69 133 189 0 63 194 81 156 117 91 113 148 185 162 218 217 217 217 9 92 219 163 236 225 254 71 152 74 89 31 172 65 253 142 202 52 100 136 110 183 215 235 17 114 36 55 19 215 143 45 226 97 255 134 173 208 156 178 8 54 89 58 232 9 225 224 205 206 115 120 13 246 82 213 221 247 117 135 138 35 218 236 76 77 213 4 192 17 101 118 158 35 228 111 56 194 158 226 42 175 190 126 32 151 112 72 245 231 8 185 16 106 191 86 255 167 95 201 154 170 56 6 216 24 131 56 28 162 236 20 221 103 137 85 121 245 137 189 191 186 254 129 195 217 207 236 32 175 100 66 9 135 142 112 124 168 68 35 56 120 243 41 31 213 29 81 133 210 234 139 253 42 159 211 105 29 210 144 46 215 50 157 102 239 223 215 233 219 108 5 67 62 198 68 69 178 7 156 76 165 215 89 247 63 9 98 254 195 47 251 7 76 195 48 14 185 218 93 49 88 39 222 145 117 158 85 196 17 168 211 8 192 65 17 109 198 46 249 55 175 14 234 69 213 145 223 176 232 22 237 22 71 142 212 159 169 113 217 145 247 225 208 12 112 228 5 25 185 194 16 31 118 71 233 32 31 90 31 206 254 3 86 47 59 251 1 28 151 156 253 16 142 97 103 63 196 235 176 179 31 196 161 189 224 236 123 249 248 167 179 239 35 246 35 103 63 136 99 216 217 247 194 184 226 236 7 112 104 7 56 251 104 179 44 85 125 168 211 56 59 48 155 230 6 102 65 198 12 212 231 150 30 157 206 102 58 173 253 225 200 63 104 252 203 142 252 172 194 106 6 56 242 82 151 103 193 124 161 126 209 13 255 114 228 113 127 119 84 8 244 156 114 140 56 170 1 28 151 156 208 16 31 195 78 104 16 199 136 210 247 226 184 162 244 3 56 162 204 3 170 56 10 113 160 48 12 89 247 171 10 20 193 225 118 245 203 98 247 159 143 179 157 70 56 188 21 56 204 117 22 71 65 69 205 32 142 88 123 223 184 200 69 69 28 101 183 219 112 126 73 11 32 254 170 172 163 144 45 214 241 231 113 160 146 185 251 61 235 88 194 65 227 153 130 205 91 235 115 185 223 99 63 172 28 216 241 210 93 154 0 188 94 85 142 136 167 31 248 55 48 130 99 80 161 7 251 44 162 172 129 254 54 27 9 29 78 60 178 4 127 207 133 217 207 152 26 111 181 167 86 172 237 155 207 61 31 136 144 163 11 143 83 150 26 47 65 141 160 253 210 3 93 49 158 102 135 187 250 162 59 67 243 184 212 41 106 54 30 244 152 97 71 232 125 27 232 47 76 65 159 143 37 24 234 223 126 46 120 219 201 233 226 43 65 120 186 146 167 167 173 17 231 201 162 139 32 54 76 62 210 153 35 137 149 146 95 30 182 28 6 136 98 195 214 76 93 241 124 177 123 44 255 189 190 210 242 137 59 239 148 3 162 159 253 96 212 110 8 106 227 4 136 226 67 178 200 133 247 38 80 144 86 118 9 35 180 178 77 11 26 32 140 115 227 145 71 218 193 112 34 198 168 187 49 74 236 14 191 171 86 118 121 4 186 187 2 174 123 254 178 244 65 147 36 182 145 202 135 250 64 40 177 147 253 161 93 33 180 41 36 18 217 233 69 197 170 183 177 124 9 199 244 155 241 9 158 38 54 27 174 117 208 184 239 186 160 222 141 19 164 93 184 115 97 4 16 111 192 90 56 27 31 250 52 95 34 33 197 86 144 153 149 34 164 100 166 23 167 227 213 177 212 86 156 162 181 210 2 134 200 95 3 138 158 221 58 186 164 131 47 77 124 13 159 36 115 63 5 222 87 194 254 50 38 249 252 248 75 113 10 204 44 42 46 156 142 21 191 222 13 39 54 157 253 162 99 174 233 246 31 3 109 109 192 197 22 0 183 222 7 244 159 102 60 229 114 215 3 205 4 210 222 2 168 251 56 124 10 175 91 40 55 166 24 170 240 74 91 53 96 234 181 128 88 185 92 202 233 255 8 159 198 28 42 47 192 253 157 64 155 6 158 167 156 248 39 120 130 249 116 140 154 123 224 107 76 221 136 131 237 176 25 115 108 199 198 226 27 225 251 152 171 167 92 218 15 96 59 230 190 64 185 113 95 134 221 72 19 238 72 0 184 233 113 232 4 152 248 8 229 150 167 2 242 61 177 137 225 88 8 135 113 241 10 227 172 164 148 58 99 50 238 237 193 127 175 207 112 99 49 119 51 229 146 231 114 201 152 163 209 12 88 253 36 135 11 24 166 36 81 46 126 9 183 16 115 108 101 200 156 106 14 103 5 167 176 223 176 166 53 156 29 217 101 51 127 101 7 184 50 204 97 167 1 8 123 57 52 28 83 153 233 40 254 60 135 179 52 211 230 83 46 99 62 119 63 174 163 96 51 148 169 102 110 61 46 222 97 219 111 22 127 130 67 30 103 220 196 154 174 157 182 106 204 100 53 38 6 41 199 179 189 11 70 7 186 24 201 25 16 176 111 101 151 215 147 104 23 171 92 78 193 38 250 78 252 58 198 95 63 225 135 234 159 244 177 145 7 33 12 241 108 188 160 171 226 104 26 64 177 121 43 165 79 74 158 184 200 48 10 123 22 170 104 12 141 84 133 75 134 243 165 162 223 227 242 84 201 186 172 74 87 192 235 143 206 246 122 2 241 97 166 178 234 3 126 81 54 218 234 228 128 183 86 184 69 90 117 175 215 95 41 27 210 157 78 9 249 114 136 129 106 89 87 44 250 171 164 128 33 163 206 229 174 12 21 140 45 177 225 138 17 41 80 231 27 77 179 170 33 98 76 148 203 116 201 72 181 84 91 129 20 196 69 6 89 89 157 48 183 234 8 70 152 168 80 186 139 39 91 182 29 139 141 102 255 197 66 192 40 119 28 88 100 103 133 158 82 56 16 196 234 133 0 26 212 209 104 228 207 35 185 13 56 149 213 133 245 255 0 0 0 13 0 235 177 139 90 230 40 155 22 251 156 88 175 93 94 231 102 13 204 147 175 125 94 167 76 59 143 217 231 192 240 108 61 233 227 55 195 231 21 254 235 219 228 247 81 114 251 36 131 237 145 201 38 131 77 126 147 209 150 236 99 242 87 235 246 147 223 228 55 185 61 32 191 201 38 155 252 38 183 201 96 147 77 54 217 100 147 77 56 121 222 49 194 234 200 111 242 219 35 183 201 109 6 220 91 155 201 41 64 52 185 158 3 230 227 55 219 71 142 185 41 153 112 130 134 77 201 94 195 3 95 64 57 232 104 194 52 117 66 72 0 190 155 9 217 253 22 124 71 160 23 133 239 114 248 246 145 32 217 15 223 205 196 190 186 149 97 140 109 250 3 212 132 100 63 238 98 181 252 5 56 94 221 2 30 96 209 111 29 249 32 116 126 239 129 47 223 37 193 59 117 132 132 206 195 171 181 240 71 126 124 131 116 146 117 147 152 128 33 122 119 234 158 200 156 251 225 131 217 198 231 91 187 252 231 156 160 191 199 221 183 151 200 87 130 23 119 245 125 130 23 2 184 35 193 163 192 196 29 178 140 144 154 17 96 161 133 41 132 106 4 41 166 191 82 205 112 84 43 93 159 200 143 142 63 1 240 123 66 149 176 218 155 163 210 238 111 104 140 134 35 81 75 75 217 15 167 81 176 110 201 222 30 253 42 76 17 202 190 253 102 3 212 109 216 223 138 117 69 246 119 78 16 234 82 168 67 80 159 192 119 173 80 191 1 245 54 172 159 194 122 59 214 59 176 46 201 126 225 183 63 129 250 36 246 35 88 111 249 222 206 174 235 80 31 130 122 4 234 195 88 55 97 237 241 75 130 172 83 97 247 55 107 186 250 47 122 201 209 37 47 248 253 7 163 115 253 61 253 178 206 26 187 63 162 36 26 236 107 70 98 142 231 99 243 252 86 141 110 249 43 237 87 222 216 54 76 249 241 249 46 71 30 125 222 95 192 121 254 199 255 223 200 199 192 222 58 79 223 213 140 255 70 205 82 19 142 105 93 43 72 252 201 121 254 151 174 224 125 70 247 171 25 239 77 67 16 2 12 69 47 218 91 163 228 189 25 121 89 58 131 255 15 200 159 168 205 121 186 175 98 252 159 48 175 217 142 150 184 90 116 246 23 103 116 255 18 242 179 100 132 191 7 74 2 165 192 210 6 79 231 144 207 128 231 83 170 51 104 90 15 105 38 199 37 207 187 209 207 5 191 43 70 111 126 68 251 59 71 223 30 251 8 234 93 217 15 223 220 61 197 187 224 187 14 245 177 192 251 106 196 243 120 95 5 252 222 128 186 26 107 95 7 191 14 252 6 88 60 232 233 96 61 211 65 76 181 52 69 111 48 146 145 214 198 214 168 105 57 247 21 241 178 72 215 200 135 208 67 13 246 133 255 29 193 250 233 209 95 125 56 238 135 207 141 126 183 133 246 95 197 247 66 71 7 241 157 151 151 132 174 132 127 190 142 250 63 133 239 219 103 124 245 0 206 35 124 230 34 226 70 17 215 243 219 243 56 71 25 226 174 196 254 57 124 127 25 235 87 178 183 127 250 181 123 186 221 131 252 94 64 60 223 175 159 159 247 237 110 228 71 232 123 63 242 225 235 250 75 129 97 80 101 157 167 235 114 174 107 51 99 36 231 118 25 159 147 254 246 44 242 36 116 157 211 177 167 219 189 168 215 156 126 125 157 238 67 28 79 159 245 248 78 248 166 167 83 79 143 207 96 157 211 163 175 195 207 244 119 41 112 25 212 179 195 211 31 236 71 64 127 103 52 35 105 14 206 10 176 115 70 127 57 127 20 122 138 97 189 102 116 236 219 180 142 35 142 159 67 132 238 60 157 157 198 121 207 102 223 223 254 173 123 58 235 122 156 87 188 92 122 6 251 235 102 242 203 99 125 77 237 215 92 125 105 118 226 209 254 169 97 201 141 253 26 242 91 99 247 183 235 73 230 223 115 28 127 126 169 14 126 33 127 255 155 92 248 189 182 228 5 191 223 33 116 179 94 225 241 187 202 238 239 210 44 39 163 232 16 3 250 52 67 205 113 28 150 252 122 188 126 198 163 199 223 252 190 235 137 164 97 61 90 118 83 32 172 69 46 20 58 204 229 189 77 88 175 149 57 101 51 246 55 98 237 233 86 228 17 47 231 213 98 189 1 235 245 50 95 136 61 137 208 177 183 31 105 38 45 203 254 8 53 95 207 187 251 174 202 200 153 112 103 198 112 180 148 90 223 166 93 41 240 15 87 3 236 231 205 33 242 72 36 214 182 53 92 31 141 134 231 198 32 29 51 115 32 31 161 136 226 244 205 77 128 112 115 115 148 183 210 59 134 30 37 161 206 15 35 252 96 102 142 170 180 101 246 192 2 220 162 204 234 197 81 232 93 201 180 62 88 89 222 145 129 125 98 28 100 54 207 199 248 227 192 49 0 15 215 250 92 102 59 126 121 44 48 108 59 153 43 180 67 130 28 213 87 146 68 93 70 81 27 223 242 253 32 123 251 7 95 15 12 71 162 109 205 180 67 150 179 89 55 248 42 129 76 253 125 68 133 14 120 41 160 126 227 71 190 247 82 231 10 12 199 155 206 198 153 135 149 208 89 143 31 14 197 175 165 85 187 126 226 190 101 124 86 234 127 12 149 117 66 28 181 54 108 26 73 205 209 76 216 106 67 34 177 76 57 1 115 115 142 74 59 165 28 117 101 68 27 82 147 17 51 169 78 207 203 60 155 163 210 206 10 142 90 218 52 160 26 78 129 34 150 20 24 42 235 148 225 172 204 33 146 150 50 56 13 204 242 0 71 165 157 149 28 181 202 69 141 171 67 206 125 100 150 10 24 42 235 148 115 212 21 237 177 211 142 166 231 41 146 26 170 64 94 35 106 138 222 66 228 120 101 73 70 168 33 72 42 81 175 244 126 164 168 87 201 80 149 208 144 101 153 86 81 67 52 117 113 53 208 78 53 119 148 82 144 118 60 28 205 163 74 71 169 65 229 53 164 211 186 26 87 244 171 211 192 146 161 85 124 214 234 134 100 210 82 109 251 108 155 66 109 101 18 90 206 90 139 179 118 170 182 153 177 18 57 69 200 89 87 11 207 179 148 244 162 134 214 160 202 143 171 182 163 232 83 174 128 26 162 137 148 107 136 118 214 34 106 179 169 39 213 130 244 228 172 235 144 215 56 156 220 21 61 69 162 174 23 168 96 90 139 168 27 208 27 98 9 75 75 231 105 245 80 55 162 83 66 138 210 139 234 147 168 155 132 26 180 33 26 164 30 171 97 51 234 181 53 97 26 139 122 125 18 29 101 254 230 8 124 228 93 215 81 222 133 205 53 67 245 55 221 128 245 158 139 250 30 217 194 81 151 219 253 5 122 56 234 117 23 245 58 217 122 12 242 255 137 67 37 169 162 20 104 2 160 187 26 158 0 104 103 27 71 13 217 78 82 51 139 137 141 110 106 4 234 27 228 41 142 90 10 168 16 37 11 105 99 2 117 187 139 106 105 70 239 2 234 14 228 149 229 221 89 94 199 81 119 82 149 183 212 150 131 238 90 84 37 111 150 108 19 205 253 149 118 118 113 212 138 127 162 66 90 158 154 149 238 173 135 35 145 112 11 235 236 70 31 106 84 7 180 124 106 152 204 67 117 232 175 81 203 4 80 251 113 218 216 131 238 197 205 36 47 7 118 97 192 103 165 157 167 57 106 121 216 76 165 77 35 159 124 37 234 51 136 202 102 109 53 210 153 105 63 151 168 207 114 212 10 150 216 10 49 91 242 186 151 163 150 52 209 245 68 65 229 50 151 215 35 67 0 74 195 205 21 115 232 1 182 156 117 31 71 173 60 101 58 90 143 150 152 206 134 147 168 251 81 229 252 96 162 160 242 137 29 192 115 28 181 12 118 27 142 101 230 227 188 68 61 128 42 143 168 70 166 152 81 36 234 65 225 67 112 40 101 246 46 48 244 60 50 68 115 94 19 236 95 102 96 165 26 94 192 89 99 204 203 23 146 232 33 158 108 202 236 126 208 67 143 150 203 0 116 31 236 38 155 17 184 31 96 168 254 189 1 96 221 114 81 111 145 195 28 21 174 37 163 186 226 244 152 86 42 150 86 19 84 240 159 76 45 103 125 145 163 250 55 134 50 231 189 228 38 0 251 234 107 5 197 74 53 188 204 103 245 239 219 100 58 124 69 36 108 6 88 76 81 112 11 134 121 8 58 71 48 97 67 2 72 45 38 236 87 57 234 170 112 103 132 143 59 29 113 36 234 81 206 171 127 127 13 108 190 227 242 250 14 92 41 51 84 255 170 89 42 175 193 157 117 246 214 6 176 110 186 168 55 225 98 131 161 250 23 30 128 53 230 162 142 193 29 4 67 245 239 38 36 175 141 174 15 205 94 237 202 93 84 83 110 214 9 229 73 247 106 118 85 158 84 155 45 165 55 53 157 190 164 94 143 187 106 152 189 55 149 62 212 194 115 121 185 98 216 90 119 90 177 128 207 79 189 72 238 27 90 197 14 64 251 162 218 93 216 15 201 52 127 130 163 150 24 25 189 224 120 147 179 158 228 168 43 6 148 110 93 179 243 195 202 89 219 56 106 40 169 13 20 9 242 102 141 224 172 131 137 62 197 42 224 202 89 79 161 151 159 4 83 80 11 204 202 205 65 59 71 173 69 75 104 26 74 168 105 154 97 165 0 165 202 163 34 245 42 41 181 83 237 5 65 60 216 19 203 89 59 40 175 225 125 254 237 72 246 207 223 140 227 245 0 116 224 98 208 117 229 135 23 134 210 81 98 98 27 13 89 121 113 51 23 71 13 181 153 131 176 252 46 104 104 34 245 158 198 89 163 230 160 106 21 19 246 132 26 186 16 21 182 155 74 250 191 12 157 65 84 122 90 82 156 85 162 158 165 168 205 191 40 19 119 27 51 192 236 22 146 59 10 237 156 227 179 174 113 111 89 166 87 62 242 122 224 117 204 121 173 41 165 151 33 210 173 249 253 173 136 68 61 207 81 43 58 26 219 123 122 236 132 165 170 159 174 81 60 94 47 160 15 197 28 197 72 42 86 146 166 188 71 219 151 139 226 190 193 28 208 138 167 10 146 161 75 152 203 27 116 77 201 111 216 38 103 237 70 189 118 105 73 181 184 19 149 222 112 153 163 86 187 202 139 100 108 185 107 152 98 72 113 179 215 236 255 98 145 153 242 10 71 245 47 150 110 181 214 227 253 16 237 192 255 131 97 168 255 249 255 49 124 214 123 125 248 31 61 36 24 141 169 189 119 239 186 143 224 13 28 173 195 27 40 73 201 198 191 192 149 113 0 110 181 107 70 162 109 182 67 1 224 205 5 184 51 252 25 28 87 190 31 109 214 109 250 230 255 0 13 13 19 83 113 117 101 97 107 80 114 105 109 115 80 80 67 46 112 114 111 106 0 0 0 0 0 0 0 0 0 0 0 0 75 202 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 19 168 0 0 0 0 0 0 0 0 255 255 255 255 77 77 80 82 67 87 73 69 1 0 172 94 103 33 179 37 114 187 0 0 41 201 0 0 3 236 0 0 9 117 0 0 1 105 177 175 202 66 0 0 0 0 0 0 84 230 13 192 213 101 111 50 207 218 43 247 228 148 211 30 207 54 27 88 71 175 149 221 62 35 108 83 66 201 109 93 107 237 57 205 148 41 167 103 153 188 99 151 120 106 120 230 121 108 157 112 252 85 103 13 186 181 8 43 77 242 155 185 37 156 240 139 69 246 73 40 121 222 145 205 34 140 240 147 236 17 217 140 252 62 74 246 81 66 9 219 231 145 45 37 199 142 176 220 118 148 39 121 78 246 147 205 6 251 37 108 27 219 18 126 226 129 187 174 228 52 143 123 121 110 105 209 111 141 176 125 204 35 108 147 219 186 217 38 191 201 111 178 201 102 120 178 129 241 167 1 125 150 7 64 0 96 225 90 128 5 248 55 220 27 82 78 126 234 1 2 202 27 17 88 236 80 78 225 255 173 134 93 75 236 202 49 245 180 58 179 246 222 86 81 216 228 118 59 237 17 41 236 7 14 209 93 46 119 141 179 126 101 25 129 74 243 96 246 100 59 94 31 138 189 235 155 231 66 170 239 101 128 219 130 243 150 219 149 95 237 121 178 178 108 163 169 110 215 158 231 138 231 96 38 183 34 164 116 46 177 131 21 255 22 252 167 173 222 255 182 169 250 119 96 221 96 169 219 69 239 70 76 207 212 120 170 151 166 84 157 200 59 165 68 86 175 95 221 186 20 32 133 113 75 97 159 49 65 150 138 59 24 198 20 12 88 189 6 160 226 215 36 103 33 222 224 192 68 239 49 41 233 217 96 251 46 158 85 105 105 112 169 17 193 187 73 104 22 207 105 169 70 134 118 223 46 111 13 158 191 63 173 118 171 28 21 131 206 128 32 203 162 124 129 194 252 247 148 58 31 38 224 16 200 128 153 14 60 93 193 28 13 14 89 96 205 234 6 68 37 85 196 234 18 188 124 117 45 143 22 193 59 151 56 11 10 236 223 30 68 113 168 101 166 30 180 115 29 233 32 233 220 253 103 112 176 244 71 170 120 56 44 67 72 118 72 65 43 83 226 105 69 33 52 104 106 182 122 164 177 123 55 192 114 133 126 154 130 233 38 87 56 36 120 195 163 76 141 176 8 204 228 135 92 172 255 60 20 140 158 234 32 54 32 156 192 237 228 254 104 120 2 118 0 148 116 1 60 241 3 188 151 75 235 248 115 76 157 115 39 131 141 149 6 193 122 55 3 33 197 208 128 249 117 167 159 130 134 219 126 163 230 0 144 72 131 35 246 243 211 248 63 225 136 57 98 125 71 22 172 67 181 179 224 16 34 111 195 51 160 203 1 169 114 180 127 229 5 104 88 92 1 235 222 216 0 177 87 122 33 246 80 155 35 246 164 153 221 137 109 247 171 76 84 126 188 213 209 78 241 28 237 95 126 5 62 243 179 29 131 92 48 19 123 248 235 154 196 113 205 99 240 154 86 19 148 223 64 161 183 239 155 237 104 127 14 111 107 252 96 250 122 56 132 255 51 124 30 122 13 26 150 191 138 231 239 48 46 7 24 231 215 97 55 83 122 58 162 34 99 172 38 46 171 24 141 5 125 165 182 53 18 9 75 209 98 90 215 14 32 254 99 189 96 203 60 0 217 79 47 74 20 95 161 170 8 162 18 255 241 165 96 171 123 21 255 135 90 148 63 57 48 127 43 181 2 176 221 84 136 185 83 248 183 194 244 118 15 226 119 205 7 163 17 243 252 59 248 200 239 184 91 205 23 70 73 193 243 157 178 87 242 193 204 82 172 234 155 10 153 163 248 79 96 29 160 62 12 63 13 241 169 30 57 9 63 152 247 250 54 199 99 223 138 98 6 139 128 242 57 241 44 216 230 236 37 165 22 55 230 14 35 191 14 86 231 217 79 63 160 226 163 30 20 223 54 55 162 49 200 129 185 212 41 72 13 112 179 138 93 98 84 10 111 17 165 77 50 95 27 21 66 77 130 212 196 87 249 26 37 65 218 90 236 170 173 226 157 197 136 244 226 17 51 236 76 61 254 203 237 136 110 86 207 180 56 22 225 205 151 222 82 186 243 190 161 230 240 202 78 127 4 209 242 148 238 59 238 27 198 59 82 56 136 103 178 226 157 10 165 251 19 84 81 188 115 244 0 113 191 217 69 255 169 157 222 13 64 110 239 85 186 103 126 30 178 124 85 202 193 182 14 122 38 119 229 248 17 115 159 210 131 213 130 56 136 234 134 133 107 190 15 51 247 119 40 61 219 54 40 61 219 99 120 191 11 29 162 225 205 78 204 124 129 242 98 202 155 17 137 254 41 218 221 106 49 216 126 1 38 218 97 104 13 70 149 47 180 73 148 206 186 243 4 2 204 232 33 47 30 60 130 127 235 54 224 202 149 195 167 88 239 96 124 225 224 82 224 142 46 130 220 18 183 187 234 70 59 68 205 176 36 18 9 232 3 240 112 185 170 86 36 113 50 209 8 248 149 65 234 142 84 250 11 24 46 119 85 185 78 0 143 122 39 154 160 30 128 174 53 181 53 195 4 63 228 225 41 91 235 49 216 27 91 188 125 125 125 64 122 30 99 23 143 62 202 92 202 241 176 31 253 219 80 106 89 33 4 2 225 112 136 175 16 3 204 92 63 82 196 193 209 93 251 160 51 191 8 246 20 247 210 43 148 7 103 247 142 241 181 184 234 213 206 118 164 58 84 30 70 187 183 160 64 23 128 135 193 238 29 19 228 239 60 226 61 197 13 93 35 163 68 174 242 16 219 198 4 185 28 106 189 182 252 183 94 173 253 60 198 213 152 134 60 220 130 236 21 2 23 133 85 117 141 8 195 207 60 255 228 225 29 85 54 169 43 109 66 41 15 19 229 241 129 19 77 241 224 236 231 250 168 129 48 212 136 247 93 169 87 121 204 167 77 40 67 137 28 100 62 205 238 191 231 211 107 157 213 19 125 140 149 130 148 143 119 50 77 97 111 235 255 44 136 245 31 146 44 141 55 13 147 60 228 150 64 99 18 39 211 93 118 73 85 228 17 109 213 9 224 65 195 104 199 42 249 143 174 110 90 139 106 71 190 172 104 149 126 75 71 142 210 95 192 184 214 145 143 240 208 13 232 200 171 87 84 242 19 122 184 220 245 73 61 244 62 58 251 247 85 189 214 217 143 243 184 218 217 79 240 152 236 236 39 116 157 236 236 147 60 244 23 58 251 97 61 254 213 217 143 8 251 97 103 159 228 49 217 217 15 211 184 222 217 143 243 208 15 232 236 141 118 89 108 254 192 167 1 58 111 61 141 83 17 139 96 197 124 244 231 174 65 159 46 103 62 173 255 209 145 191 95 248 215 58 242 139 14 171 27 208 145 215 251 66 183 127 156 111 43 90 246 239 142 220 252 143 142 10 137 142 57 199 84 71 53 206 227 106 39 52 161 199 100 39 148 228 49 229 244 195 60 174 59 253 56 15 131 125 220 21 211 144 7 26 195 68 235 126 195 129 18 60 2 190 81 91 28 120 249 184 88 105 148 71 184 17 7 153 46 242 168 110 244 39 121 164 187 70 198 69 174 56 98 154 203 229 172 208 5 40 138 191 189 85 164 161 90 172 226 47 241 64 39 11 140 246 172 217 148 7 29 77 228 157 225 96 196 23 120 87 253 184 115 96 197 139 27 117 1 116 189 225 28 137 158 126 252 109 96 138 71 210 161 147 117 150 112 214 232 104 153 77 133 9 167 253 88 130 219 124 121 238 49 197 31 110 9 5 133 224 200 108 234 165 64 134 132 30 56 156 48 212 121 9 90 8 250 47 53 208 31 51 232 220 108 127 93 12 100 232 44 42 173 20 53 155 9 169 152 97 63 237 126 15 164 94 158 0 190 20 203 49 212 215 126 18 91 123 118 178 246 122 80 62 253 201 243 147 198 200 243 44 104 17 164 199 197 71 57 43 68 161 73 148 228 201 150 195 2 6 54 104 172 225 147 151 7 70 210 223 173 43 61 159 184 231 30 57 42 72 236 129 81 191 33 104 25 219 192 192 105 182 72 226 59 3 104 80 175 76 152 101 55 219 50 160 3 32 185 254 131 226 100 6 234 72 247 15 12 69 134 182 23 66 242 5 101 95 200 45 241 52 227 139 250 54 255 101 225 129 46 73 236 71 151 39 3 117 112 30 164 188 164 214 243 215 23 101 61 130 194 125 167 11 26 222 182 64 189 139 235 223 122 49 17 156 5 177 112 54 94 251 116 95 194 150 239 172 46 45 203 231 243 75 75 60 37 120 116 175 116 122 242 245 86 74 192 162 45 97 209 98 112 175 140 41 247 216 177 91 126 139 87 242 200 79 129 139 172 97 175 140 185 17 9 159 20 231 192 194 90 79 205 60 68 252 218 0 126 122 9 123 162 99 93 211 221 63 2 186 181 33 147 230 238 186 31 232 59 205 12 154 171 124 4 232 76 32 221 91 0 173 55 195 167 241 184 151 230 166 121 160 25 143 116 163 4 204 93 0 200 149 84 210 92 234 239 225 179 152 251 18 205 61 216 7 116 211 192 79 104 78 248 3 60 197 250 116 12 255 102 248 42 115 55 140 141 127 134 103 48 199 182 90 84 126 12 190 135 185 54 154 91 220 7 63 198 220 227 52 231 136 192 97 124 136 123 150 230 196 37 208 135 178 34 47 128 200 243 128 122 231 50 174 107 235 224 36 192 172 233 52 23 220 66 43 99 22 110 143 0 184 99 51 201 198 28 123 119 226 223 38 121 152 99 51 142 41 132 224 2 134 217 76 227 85 143 147 79 98 174 134 230 138 238 36 56 43 56 155 117 15 203 190 72 92 136 202 182 169 52 236 32 13 152 195 74 195 133 19 143 18 124 8 231 217 83 237 180 151 8 206 210 204 181 51 30 223 36 15 2 124 148 189 79 167 244 146 71 16 137 213 214 150 227 4 117 156 87 196 138 238 0 221 170 177 128 97 220 18 163 57 142 237 93 200 114 227 174 31 209 27 229 177 110 101 95 56 148 227 18 154 125 94 222 41 68 134 158 142 241 233 39 126 81 125 73 207 78 92 208 56 100 178 241 130 126 196 12 58 128 226 12 55 137 119 138 33 115 98 24 133 93 211 16 179 180 145 170 56 100 60 95 47 72 33 95 168 89 54 149 53 249 162 97 201 88 30 14 69 51 227 74 149 181 69 37 65 206 114 182 202 209 112 144 95 37 110 221 18 150 154 100 75 137 215 43 162 94 110 33 218 34 155 60 130 212 44 70 45 43 90 125 129 38 13 48 125 141 19 87 140 136 209 214 72 6 157 85 213 132 177 210 92 169 79 70 169 197 96 35 74 96 78 12 178 50 156 184 182 234 8 70 92 40 45 221 175 147 179 220 133 96 25 236 93 76 35 70 115 103 136 37 118 86 164 210 20 14 4 49 60 141 160 69 29 141 70 253 66 98 192 130 83 89 253 92 255 15 0 65 109 177 88 4 208 212 91 0 174 148 122 119 43 86 175 94 69 189 85 202 89 129 206 76 18 155 13 39 227 253 135 63 174 102 224 166 3 45 206 156 146 228 188 37 220 174 127 244 27 51 45 11 253 119 51 245 143 222 209 101 91 239 88 252 31 243 254 179 223 254 236 237 179 238 142 125 8 253 217 153 144 189 93 185 114 213 7 149 94 142 101 252 193 188 177 153 188 92 247 189 14 39 111 52 15 121 207 231 52 253 131 211 157 221 36 228 85 113 103 235 43 118 90 60 139 75 197 117 144 69 192 185 16 122 35 172 1 93 1 48 118 249 246 88 111 220 232 137 135 187 195 201 240 22 51 113 76 113 219 95 55 166 102 230 185 4 92 79 255 109 143 140 186 166 90 163 178 184 161 161 178 184 54 188 233 175 13 156 205 255 180 135 94 19 77 154 241 142 80 155 249 95 106 36 114 255 169 135 183 46 148 236 250 231 10 76 244 255 240 211 30 217 246 94 27 122 163 201 112 183 249 159 151 139 87 230 236 38 226 132 255 98 151 183 30 247 3 159 150 74 36 123 55 217 4 116 159 211 225 23 151 116 235 239 63 211 172 165 234 26 106 87 218 4 92 233 174 191 216 211 173 191 191 242 166 193 212 119 222 132 59 157 250 139 43 57 177 33 53 125 189 191 249 103 144 84 115 245 186 225 127 20 104 234 191 12 122 67 147 217 169 254 166 182 208 67 176 61 244 234 60 19 126 175 254 97 100 146 253 183 170 175 77 36 237 0 244 108 86 177 111 84 158 204 134 149 145 132 237 17)
! !


!SampledSound methodsFor: 'initialization' stamp: 'jm 3/24/1999 12:01'!
setPitch: pitchNameOrNumber dur: d loudness: vol
	"Used to play scores using the default sample table."
	"(SampledSound pitch: 880.0 dur: 1.5 loudness: 0.6) play"

	| p |
	super setPitch: pitchNameOrNumber dur: d loudness: vol.
	p _ self nameOrNumberToPitch: pitchNameOrNumber.
	samples _ DefaultSampleTable.
	samplesSize _ samples size.
	initialCount _ (d * self samplingRate asFloat) rounded.
	originalSamplingRate _
		((self samplingRate asFloat * p asFloat) / NominalSamplePitch asFloat) asInteger.
	self loudness: vol.
	self reset.
! !

!SampledSound methodsFor: 'initialization' stamp: 'jm 3/29/1999 10:41'!
setSamples: anArray samplingRate: rate
	"Set my samples array to the given array with the given nominal sampling rate. Altering the rate parameter allows the sampled sound to be played back at different pitches."
	"Assume: anArray is either a SoundBuffer or a collection of signed 16-bit sample values."
	"(SampledSound
		samples: SampledSound coffeeCupClink
		samplingRate: 5000) play"

	"copy the array into a SoundBuffer if necessary"
	anArray class isWords
		ifTrue: [samples _ anArray]
		ifFalse: [samples _ SoundBuffer fromArray: anArray].

	samplesSize _ samples size.
	originalSamplingRate _ rate.
	initialCount _ (samplesSize * self samplingRate) // originalSamplingRate.
	self loudness: 1.0.
	self reset.
! !


!SoundCodec methodsFor: 'compress/decompress' stamp: 'jm 3/30/1999 08:03'!
decompressSound: aCompressedSound
	"Decompress the entirety of the given compressed sound with this codec and answer the resulting sound."

	| channels sound |
	channels _ aCompressedSound channels
		collect: [:compressed | self decodeCompressedData: compressed].
	'SampledSound' = aCompressedSound soundClassName ifTrue: [
		sound _ SampledSound
			samples: channels first
			samplingRate: (aCompressedSound samplingRate).
		sound loudness: aCompressedSound gain.
		^ sound].
	'LoopedSampledSound' = aCompressedSound soundClassName ifTrue: [
		aCompressedSound loopLength = 0
			ifTrue: [
				sound _ LoopedSampledSound
					unloopedSamples: channels first
					pitch: aCompressedSound perceivedPitch
					samplingRate: aCompressedSound samplingRate]
			ifFalse: [
				sound _ LoopedSampledSound
					samples: channels first
					loopEnd: aCompressedSound loopEnd
					loopLength: aCompressedSound loopLength
					pitch: aCompressedSound perceivedPitch
					samplingRate: aCompressedSound samplingRate].
		channels size > 1 ifTrue: [sound rightSamples: channels last].
		sound
			firstSample: aCompressedSound firstSample;
			gain: aCompressedSound gain.
		sound
			setPitch: 100.0
			dur: (channels first size / aCompressedSound samplingRate)
			loudness: 1.0.
		^ sound].
	self error: 'unknown sound class'.
! !


!ADPCMCodec methodsFor: 'all' stamp: 'jm 3/30/1999 08:56'!
decodeFlash: aByteArray sampleCount: sampleCount stereo: stereoFlag

	| bits |
	encodedBytes _ aByteArray.
	byteIndex _ 0.
	bitPosition _ 0.
	currentByte _ 0.
	bits _ 2 + (self nextBits: 2).  "bits per sample"
	self initializeForBitsPerSample: bits samplesPerFrame: 4096.
	stereoFlag
		ifTrue: [
			self resetForStereo.
			samples _ SoundBuffer newMonoSampleCount: sampleCount.
			rightSamples _ SoundBuffer newMonoSampleCount: sampleCount.
			sampleIndex _ 0.
			self privateDecodeStereo: sampleCount.
			^ Array with: samples with: rightSamples]
		ifFalse: [
			samples _ SoundBuffer newMonoSampleCount: sampleCount.
			sampleIndex _ 0.
			self privateDecodeMono: sampleCount.
			^ Array with: samples].
! !


AIFFFileReader removeSelector: #play!
AIFFFileReader removeSelector: #playLoudness:!
AbstractSound removeSelector: #setLoudness:!
SampledInstrument removeSelector: #tempTest!
